@@ -18,7 +18,6 @@ class CZNC;
 class CUser;
 class CIRCSock;
 class CClient;
-class CClientTimeout;
 // !Forward Declarations
 
 class CAuthBase {
@@ -36,7 +35,7 @@ public:
 		m_pSock = pSock;
 	}
 
-	void AcceptLogin(CUser& User) { AcceptedLogin(User); }
+	void AcceptLogin(CUser& User);
 	void RefuseLogin(const CString& sReason);
 
 	const CString& GetUsername() const { return m_sUsername; }
@@ -44,6 +43,10 @@ public:
 	Csock *GetSocket() const { return m_pSock; }
 	CString GetRemoteIP() const;
 
+	// Invalidate this CAuthBase instance which means it will no longer use
+	// m_pSock and AcceptLogin() or RefusedLogin() will have no effect.
+	virtual void Invalidate();
+
 protected:
 	virtual void AcceptedLogin(CUser& User) = 0;
 	virtual void RefusedLogin(const CString& sReason) = 0;
@@ -60,7 +63,7 @@ public:
 	CClientAuth(CClient* pClient, const CString& sUsername, const CString& sPassword);
 	virtual ~CClientAuth() {}
 
-	void SetClient(CClient* pClient) { m_pClient = pClient; }
+	void Invalidate() { m_pClient = NULL; CAuthBase::Invalidate(); }
 	void AcceptedLogin(CUser& User);
 	void RefusedLogin(const CString& sReason);
 private:
@@ -70,9 +73,8 @@ protected:
 
 class CClient : public CZNCSock {
 public:
-	CClient(const CString& sHostname, unsigned short uPort, int iTimeout = 60) : CZNCSock(sHostname, uPort, iTimeout) {
+	CClient(const CString& sHostname, unsigned short uPort) : CZNCSock(sHostname, uPort) {
 		m_pUser = NULL;
-		m_pTimeout = NULL;
 		m_pIRCSock = NULL;
 		m_bGotPass = false;
 		m_bGotNick = false;
@@ -84,7 +86,9 @@ public:
 		// a little more gentle ;)
 		SetMaxBufferThreshold(1024);
 
-		StartLoginTimeout();
+		// Disable all timeout types. The socket will now time out in 60
+		// seconds, no matter what. AcceptLogin() fixes this up.
+		SetTimeout(60, 0);
 
 		SetNick("unknown-nick");
 	}
@@ -93,8 +97,6 @@ public:
 
 	void AcceptLogin(CUser& User);
 	void RefuseLogin(const CString& sReason);
-	void StartLoginTimeout();
-	void LoginTimeout();
 
 	CString GetNick(bool bAllowIRCNick = true) const;
 	CString GetNickMask() const;
@@ -121,6 +123,7 @@ public:
 	void HelpUser();
 	void AuthUser();
 	virtual void Connected();
+	virtual void Timeout();
 	virtual void Disconnected();
 	virtual void ConnectionRefused();
 	virtual void ReachedMaxBuffer();
@@ -141,7 +144,6 @@ protected:
 	CString		m_sUser;
 	CIRCSock*	m_pIRCSock;
 	CSmartPtr<CAuthBase>	m_spAuth;
-	CClientTimeout*	m_pTimeout;
 };
 
 #endif // !_CLIENT_H
