@@ -204,24 +204,27 @@ void CClient::UserCommand(CString& sLine) {
 		PutStatus("Cleared MOTD");
 	} else if (m_pUser->IsAdmin() && sCommand.Equals("BROADCAST")) {
 		CZNC::Get().Broadcast(sLine.Token(1, true));
-	} else if (m_pUser->IsAdmin() && sCommand.Equals("SHUTDOWN")) {
+	} else if (m_pUser->IsAdmin() && (sCommand.Equals("SHUTDOWN") || sCommand.Equals("RESTART"))) {
+		bool bRestart = sCommand.Equals("RESTART");
 		CString sMessage = sLine.Token(1, true);
+		bool bForce = false;
 
-		if (sMessage.empty()) {
-			sMessage = "ZNC is being shutdown NOW!!";
+		if (sMessage.Token(0).Equals("FORCE")) {
+			bForce = true;
+			sMessage = sMessage.Token(1, true);
 		}
 
-		CZNC::Get().Broadcast(sMessage);
-		throw CException(CException::EX_Shutdown);
-	} else if (m_pUser->IsAdmin() && sCommand.Equals("RESTART")) {
-		CString sMessage = sLine.Token(1, true);
-
 		if (sMessage.empty()) {
-			sMessage = "ZNC is being restarted NOW!!";
+			sMessage = (bRestart ? "ZNC is being restarted NOW!" : "ZNC is being shut down NOW!");
 		}
 
-		CZNC::Get().Broadcast(sMessage);
-		throw CException(CException::EX_Restart);
+		if(!CZNC::Get().WriteConfig() && !bForce) {
+			PutStatus("ERROR: Writing config file to disk failed! Aborting. Use " +
+				sCommand.AsUpper() + " FORCE to ignore.");
+		} else {
+			CZNC::Get().Broadcast(sMessage);
+			throw CException(bRestart ? CException::EX_Restart : CException::EX_Shutdown);
+		}
 	} else if (sCommand.Equals("JUMP") || sCommand.Equals("CONNECT")) {
 		if (!m_pUser->HasServers()) {
 			PutStatus("You don't have any servers added.");
@@ -329,20 +332,20 @@ void CClient::UserCommand(CString& sLine) {
 		}
 	} else if (sCommand.Equals("REMSERVER") || sCommand.Equals("DELSERVER")) {
 		CString sServer = sLine.Token(1);
+		unsigned short uPort = sLine.Token(2).ToUShort();
+		CString sPass = sLine.Token(3);
 
 		if (sServer.empty()) {
-			PutStatus("Usage: RemServer <host>");
+			PutStatus("Usage: RemServer <host> [port] [pass]");
 			return;
 		}
 
-		const vector<CServer*>& vServers = m_pUser->GetServers();
-
-		if (vServers.size() <= 0) {
+		if (!m_pUser->HasServers()) {
 			PutStatus("You don't have any servers added.");
 			return;
 		}
 
-		if (m_pUser->DelServer(sServer)) {
+		if (m_pUser->DelServer(sServer, uPort, sPass)) {
 			PutStatus("Server removed");
 		} else {
 			PutStatus("No such server");
@@ -635,8 +638,9 @@ void CClient::UserCommand(CString& sLine) {
 
 #ifdef _MODULES
 		CModInfo ModInfo;
-		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
-			PutStatus("Unable to find modinfo [" + sMod + "]");
+		CString sRetMsg;
+		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod, sRetMsg)) {
+			PutStatus("Unable to find modinfo [" + sMod + "] [" + sRetMsg + "]");
 			return;
 		}
 
@@ -675,30 +679,19 @@ void CClient::UserCommand(CString& sLine) {
 			return;
 		}
 #ifdef _MODULES
-		CModInfo ModInfo;
-		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
-			PutStatus("Unable to find modinfo for [" + sMod + "]");
-			return;
-		}
-
-		bool bGlobal = ModInfo.IsGlobal();
-
-		if (bGlobal && !m_pUser->IsAdmin()) {
-			PutStatus("Unable to unload global module [" + sMod + "] Access Denied.");
-			return;
-		}
-
 		if (sMod.empty()) {
 			PutStatus("Usage: UnloadMod <module>");
 			return;
 		}
 
 		CString sModRet;
+		bool b;
 
-		if (bGlobal) {
-			CZNC::Get().GetModules().UnloadModule(sMod, sModRet);
-		} else {
-			m_pUser->GetModules().UnloadModule(sMod, sModRet);
+		// First, try to unload the user module
+		b = m_pUser->GetModules().UnloadModule(sMod, sModRet);
+		if (!b && m_pUser->IsAdmin()) {
+			// If that failed and the user is an admin, try to unload a global module
+			b = CZNC::Get().GetModules().UnloadModule(sMod, sModRet);
 		}
 
 		PutStatus(sModRet);
@@ -719,8 +712,9 @@ void CClient::UserCommand(CString& sLine) {
 		}
 #ifdef _MODULES
 		CModInfo ModInfo;
-		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
-			PutStatus("Unable to find modinfo for [" + sMod + "]");
+		CString sRetMsg;
+		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod, sRetMsg)) {
+			PutStatus("Unable to find modinfo for [" + sMod + "] [" + sRetMsg + "]");
 			return;
 		}
 
