@@ -81,7 +81,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 		PutIRC("PONG " + sLine.substr(5));
 		m_pUser->PutUser(sLine);
 		return;
-	} else if (sLine.Equals("PONG ", false, 5) && sLine.Token(3).Equals(":ZNC")) {
+	} else if (sLine.Token(1).Equals("PONG") && sLine.Token(3).Equals(":ZNC")) {
 		// We asked for this so don't forward the reply to clients.
 		return;
 	} else if (sLine.Equals("ERROR ", false, 6)) {
@@ -139,7 +139,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			}
 			case 5:
 				ParseISupport(sRest);
-				m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+				m_pUser->UpdateExactRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 				break;
 			case 2:
 			case 3:
@@ -153,46 +153,31 @@ void CIRCSock::ReadLine(const CString& sData) {
 			case 266:	// global users
 				m_pUser->UpdateRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 				break;
-			case 422:	// MOTD File is missing
-			case 375: 	// begin motd
-				m_pUser->ClearMotdBuffer();
-			case 372:	// motd
-			case 376:	// end motd
-				m_pUser->AddMotdBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+			case 305:
+				m_pUser->SetIRCAway(false);
 				break;
-			case 470: {
-				// :irc.unreal.net 470 mynick [Link] #chan1 has become full, so you are automatically being transferred to the linked channel #chan2
-				// :mccaffrey.freenode.net 470 mynick #electronics ##electronics :Forwarding to another channel
-
-				// freenode style numeric
+			case 306:
+				m_pUser->SetIRCAway(true);
+				break;
+			case 324: {	// MODE
+				sRest.Trim();
 				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
-				if (!pChan) {
-					// unreal style numeric
-					pChan = m_pUser->FindChan(sRest.Token(1));
-				}
-				if (pChan) {
-					pChan->Disable();
-					m_pUser->PutStatus("Channel [" + pChan->GetName() + "] is linked to "
-							"another channel and was thus disabled.");
-				}
-				break;
-			}
-			case 437:
-				// :irc.server.net 437 * badnick :Nick/channel is temporarily unavailable
-				// :irc.server.net 437 mynick badnick :Nick/channel is temporarily unavailable
-				// :irc.server.net 437 mynick badnick :Cannot change nickname while banned on channel
-				if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
-					break;
-			case 432: // :irc.server.com 432 * nick :Erroneous Nickname: Illegal characters
-			case 433: {
-				CString sBadNick = sRest.Token(0);
 
-				if (!m_bAuthed) {
-					SendAltNick(sBadNick);
-					return;
+				if (pChan) {
+					pChan->SetModes(sRest.Token(1, true));
 				}
-				break;
 			}
+				break;
+			case 329: {
+				sRest.Trim();
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+
+				if (pChan) {
+					unsigned long ulDate = sLine.Token(4).ToULong();
+					pChan->SetCreationDate(ulDate);
+				}
+			}
+				break;
 			case 331: {
 				// :irc.server.com 331 yournick #chan :No topic is set.
 				CChan* pChan = m_pUser->FindChan(sLine.Token(3));
@@ -254,25 +239,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				break;
 			}
-			case 324: {	// MODE
-				sRest.Trim();
-				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
-
-				if (pChan) {
-					pChan->SetModes(sRest.Token(1, true));
-				}
-			}
-				break;
-			case 329: {
-				sRest.Trim();
-				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
-
-				if (pChan) {
-					unsigned long ulDate = sLine.Token(4).ToULong();
-					pChan->SetCreationDate(ulDate);
-				}
-			}
-				break;
 			case 353: {	// NAMES
 				sRest.Trim();
 				// Todo: allow for non @+= server msgs
@@ -318,6 +284,46 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				return;	// return so we don't send them the raw twice
 			}
+			case 375: 	// begin motd
+			case 422:	// MOTD File is missing
+				m_pUser->ClearMotdBuffer();
+			case 372:	// motd
+			case 376:	// end motd
+				m_pUser->AddMotdBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+				break;
+			case 437:
+				// :irc.server.net 437 * badnick :Nick/channel is temporarily unavailable
+				// :irc.server.net 437 mynick badnick :Nick/channel is temporarily unavailable
+				// :irc.server.net 437 mynick badnick :Cannot change nickname while banned on channel
+				if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
+					break;
+			case 432: // :irc.server.com 432 * nick :Erroneous Nickname: Illegal characters
+			case 433: {
+				CString sBadNick = sRest.Token(0);
+
+				if (!m_bAuthed) {
+					SendAltNick(sBadNick);
+					return;
+				}
+				break;
+			}
+			case 470: {
+				// :irc.unreal.net 470 mynick [Link] #chan1 has become full, so you are automatically being transferred to the linked channel #chan2
+				// :mccaffrey.freenode.net 470 mynick #electronics ##electronics :Forwarding to another channel
+
+				// freenode style numeric
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+				if (!pChan) {
+					// unreal style numeric
+					pChan = m_pUser->FindChan(sRest.Token(1));
+				}
+				if (pChan) {
+					pChan->Disable();
+					m_pUser->PutStatus("Channel [" + pChan->GetName() + "] is linked to "
+							"another channel and was thus disabled.");
+				}
+				break;
+			}
 		}
 	} else {
 		CNick Nick(sLine.Token(0).LeftChomp_n());
@@ -650,15 +656,17 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 
 		if (sType.Equals("CHAT")) {
 			CNick FromNick(Nick.GetNickMask());
-			unsigned short uBNCPort = CDCCBounce::DCCRequest(FromNick.GetNick(), uLongIP, uPort, "", true, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
+			unsigned short uBNCPort = CDCCBounce::DCCRequest(FromNick.GetNick(), uLongIP, uPort, "", true, m_pUser, CUtils::GetIP(uLongIP));
 			if (uBNCPort) {
-				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC CHAT chat " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + "\001");
+				CString sIP = m_pUser->GetLocalDCCIP();
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC CHAT chat " + CString(CUtils::GetLongIP(sIP)) + " " + CString(uBNCPort) + "\001");
 			}
 		} else if (sType.Equals("SEND")) {
 			// DCC SEND readme.txt 403120438 5550 1104
-			unsigned short uBNCPort = CDCCBounce::DCCRequest(Nick.GetNick(), uLongIP, uPort, sFile, false, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
+			unsigned short uBNCPort = CDCCBounce::DCCRequest(Nick.GetNick(), uLongIP, uPort, sFile, false, m_pUser, CUtils::GetIP(uLongIP));
 			if (uBNCPort) {
-				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC SEND " + sFile + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + " " + CString(uFileSize) + "\001");
+				CString sIP = m_pUser->GetLocalDCCIP();
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC SEND " + sFile + " " + CString(CUtils::GetLongIP(sIP)) + " " + CString(uBNCPort) + " " + CString(uFileSize) + "\001");
 			}
 		} else if (sType.Equals("RESUME")) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
@@ -816,6 +824,9 @@ void CIRCSock::Connected() {
 
 	PutIRC("NICK " + sNick);
 	PutIRC("USER " + sIdent + " \"" + sIdent + "\" \"" + sIdent + "\" :" + sRealName);
+
+	// SendAltNick() needs this
+	m_Nick.SetNick(sNick);
 }
 
 void CIRCSock::Disconnected() {
@@ -830,6 +841,19 @@ void CIRCSock::Disconnected() {
 	m_pUser->ClearMotdBuffer();
 
 	ResetChans();
+
+	// send a "reset user modes" cmd to the client.
+	// otherwise, on reconnect, it might think it still
+	// had user modes that it actually doesn't have.
+	CString sUserMode;
+	for (set<unsigned char>::const_iterator it = m_scUserModes.begin(); it != m_scUserModes.end(); it++) {
+		sUserMode += *it;
+	}
+	if (!sUserMode.empty()) {
+		m_pUser->PutUser(":" + m_pUser->GetIRCNick().GetNickMask() + " MODE " + m_pUser->GetIRCNick().GetNick() + " :-" + sUserMode);
+	}
+
+	// also clear the user modes in our space:
 	m_scUserModes.clear();
 }
 
@@ -989,24 +1013,34 @@ void CIRCSock::ForwardRaw353(const CString& sLine) const {
 }
 
 void CIRCSock::SendAltNick(const CString& sBadNick) {
-	const unsigned int uMax = GetMaxNickLen();
-	const CString& sConfNick = m_pUser->GetNick().Left(uMax);
-	const CString& sAltNick = m_pUser->GetAltNick().Left(uMax);
+	const CString& sLastNick = m_Nick.GetNick();
 
-	if (sBadNick.Equals(sConfNick)) {
+	// We don't know the maximum allowed nick length yet, but we know which
+	// nick we sent last. If sBadNick is shorter than that, we assume the
+	// server truncated our nick.
+	if (sBadNick.length() < sLastNick.length())
+		m_uMaxNickLen = sBadNick.length();
+
+	unsigned int uMax = m_uMaxNickLen;
+
+	const CString& sConfNick = m_pUser->GetNick();
+	const CString& sAltNick  = m_pUser->GetAltNick();
+	CString sNewNick;
+
+	if (sLastNick.Equals(sConfNick)) {
 		if ((!sAltNick.empty()) && (!sConfNick.Equals(sAltNick))) {
-			PutIRC("NICK " + sAltNick);
+			sNewNick = sAltNick;
 		} else {
-			PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
+			sNewNick = sConfNick.Left(uMax - 1) + "-";
 		}
-	} else if (sBadNick.Equals(sAltNick)) {
-		PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
-	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "-"))) {
-		PutIRC("NICK " + sConfNick.Left(uMax -1) + "|");
-	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "|"))) {
-		PutIRC("NICK " + sConfNick.Left(uMax -1) + "^");
-	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "^"))) {
-		PutIRC("NICK " + sConfNick.Left(uMax -1) + "a");
+	} else if (sLastNick.Equals(sAltNick)) {
+		sNewNick = sConfNick.Left(uMax -1) + "-";
+	} else if (sLastNick.Equals(CString(sConfNick.Left(uMax -1) + "-"))) {
+		sNewNick = sConfNick.Left(uMax -1) + "|";
+	} else if (sLastNick.Equals(CString(sConfNick.Left(uMax -1) + "|"))) {
+		sNewNick = sConfNick.Left(uMax -1) + "^";
+	} else if (sLastNick.Equals(CString(sConfNick.Left(uMax -1) + "^"))) {
+		sNewNick = sConfNick.Left(uMax -1) + "a";
 	} else {
 		char cLetter = 0;
 		if (sBadNick.empty()) {
@@ -1023,9 +1057,10 @@ void CIRCSock::SendAltNick(const CString& sBadNick) {
 			return;
 		}
 
-		CString sSend = "NICK " + sConfNick.Left(uMax -1) + ++cLetter;
-		PutIRC(sSend);
+		sNewNick = sConfNick.Left(uMax -1) + ++cLetter;
 	}
+	PutIRC("NICK " + sNewNick);
+	m_Nick.SetNick(sNewNick);
 }
 
 unsigned char CIRCSock::GetPermFromMode(unsigned char uMode) const {
