@@ -235,12 +235,12 @@ CModule::CModule(ModHandle pDLL, const CString& sModName, const CString& sDataDi
 }
 
 CModule::~CModule() {
-	while (m_vTimers.size()) {
-		RemTimer(m_vTimers[0]->GetName());
+	while (!m_sTimers.empty()) {
+		RemTimer(*m_sTimers.begin());
 	}
 
-	while (m_vSockets.size()) {
-		RemSocket(m_vSockets[0]);
+	while (!m_sSockets.empty()) {
+		RemSocket(*m_sSockets.begin());
 	}
 
 	SaveRegistry();
@@ -310,8 +310,11 @@ bool CModule::AddTimer(CTimer* pTimer) {
 		return false;
 	}
 
+	if (!m_sTimers.insert(pTimer).second)
+		// Was already added
+		return true;
+
 	m_pManager->AddCron(pTimer);
-	m_vTimers.push_back(pTimer);
 	return true;
 }
 
@@ -322,24 +325,25 @@ bool CModule::AddTimer(FPTimer_t pFBCallback, const CString& sLabel, u_int uInte
 	return AddTimer(pTimer);
 }
 
+bool CModule::RemTimer(CTimer* pTimer) {
+	if (m_sTimers.erase(pTimer) == 0)
+		return false;
+	m_pManager->DelCronByAddr(pTimer);
+	return true;
+}
+
 bool CModule::RemTimer(const CString& sLabel) {
-	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
-		CTimer* pTimer = m_vTimers[a];
-
-		if (pTimer->GetName().Equals(sLabel)) {
-			m_vTimers.erase(m_vTimers.begin() +a);
-			m_pManager->DelCronByAddr(pTimer);
-			return true;
-		}
-	}
-
-	return false;
+	CTimer *pTimer = FindTimer(sLabel);
+	if (!pTimer)
+		return false;
+	return RemTimer(pTimer);
 }
 
 bool CModule::UnlinkTimer(CTimer* pTimer) {
-	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
-		if (pTimer == m_vTimers[a]) {
-			m_vTimers.erase(m_vTimers.begin() +a);
+	set<CTimer*>::iterator it;
+	for (it = m_sTimers.begin(); it != m_sTimers.end(); ++it) {
+		if (pTimer == *it) {
+			m_sTimers.erase(it);
 			return true;
 		}
 	}
@@ -348,8 +352,9 @@ bool CModule::UnlinkTimer(CTimer* pTimer) {
 }
 
 CTimer* CModule::FindTimer(const CString& sLabel) {
-	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
-		CTimer* pTimer = m_vTimers[a];
+	set<CTimer*>::iterator it;
+	for (it = m_sTimers.begin(); it != m_sTimers.end(); ++it) {
+		CTimer* pTimer = *it;
 		if (pTimer->GetName().Equals(sLabel)) {
 			return pTimer;
 		}
@@ -359,7 +364,7 @@ CTimer* CModule::FindTimer(const CString& sLabel) {
 }
 
 void CModule::ListTimers() {
-	if (!m_vTimers.size()) {
+	if (m_sTimers.empty()) {
 		PutModule("You have no timers running.");
 		return;
 	}
@@ -370,8 +375,9 @@ void CModule::ListTimers() {
 	Table.AddColumn("Cycles");
 	Table.AddColumn("Description");
 
-	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
-		CTimer* pTimer = (CTimer*) m_vTimers[a];
+	set<CTimer*>::iterator it;
+	for (it = m_sTimers.begin(); it != m_sTimers.end(); ++it) {
+		CTimer* pTimer = *it;
 		unsigned int uCycles = pTimer->GetCyclesLeft();
 
 		Table.AddRow();
@@ -389,14 +395,15 @@ bool CModule::AddSocket(CSocket* pSocket) {
 		return false;
 	}
 
-	m_vSockets.push_back(pSocket);
+	m_sSockets.insert(pSocket);
 	return true;
 }
 
 bool CModule::RemSocket(CSocket* pSocket) {
-	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
-		if (m_vSockets[a] == pSocket) {
-			m_vSockets.erase(m_vSockets.begin() +a);
+	set<CSocket*>::iterator it;
+	for (it = m_sSockets.begin(); it != m_sSockets.end(); ++it) {
+		if (*it == pSocket) {
+			m_sSockets.erase(it);
 			m_pManager->DelSockByAddr(pSocket);
 			return true;
 		}
@@ -406,11 +413,12 @@ bool CModule::RemSocket(CSocket* pSocket) {
 }
 
 bool CModule::RemSocket(const CString& sSockName) {
-	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
-		CSocket* pSocket = m_vSockets[a];
+	set<CSocket*>::iterator it;
+	for (it = m_sSockets.begin(); it != m_sSockets.end(); ++it) {
+		CSocket* pSocket = *it;
 
 		if (pSocket->GetSockName().Equals(sSockName)) {
-			m_vSockets.erase(m_vSockets.begin() +a);
+			m_sSockets.erase(it);
 			m_pManager->DelSockByAddr(pSocket);
 			return true;
 		}
@@ -420,9 +428,10 @@ bool CModule::RemSocket(const CString& sSockName) {
 }
 
 bool CModule::UnlinkSocket(CSocket* pSocket) {
-	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
-		if (pSocket == m_vSockets[a]) {
-			m_vSockets.erase(m_vSockets.begin() +a);
+	set<CSocket*>::iterator it;
+	for (it = m_sSockets.begin(); it != m_sSockets.end(); ++it) {
+		if (pSocket == *it) {
+			m_sSockets.erase(it);
 			return true;
 		}
 	}
@@ -431,8 +440,9 @@ bool CModule::UnlinkSocket(CSocket* pSocket) {
 }
 
 CSocket* CModule::FindSocket(const CString& sSockName) {
-	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
-		CSocket* pSocket = m_vSockets[a];
+	set<CSocket*>::iterator it;
+	for (it = m_sSockets.begin(); it != m_sSockets.end(); ++it) {
+		CSocket* pSocket = *it;
 		if (pSocket->GetSockName().Equals(sSockName)) {
 			return pSocket;
 		}
@@ -442,7 +452,7 @@ CSocket* CModule::FindSocket(const CString& sSockName) {
 }
 
 void CModule::ListSockets() {
-	if (!m_vSockets.size()) {
+	if (m_sSockets.empty()) {
 		PutModule("You have no open sockets.");
 		return;
 	}
@@ -455,8 +465,9 @@ void CModule::ListSockets() {
 	Table.AddColumn("RemoteIP");
 	Table.AddColumn("RemotePort");
 
-	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
-		CSocket* pSocket = (CSocket*) m_vSockets[a];
+	set<CSocket*>::iterator it;
+	for (it = m_sSockets.begin(); it != m_sSockets.end(); ++it) {
+		CSocket* pSocket = *it;
 
 		Table.AddRow();
 		Table.SetCell("Name", pSocket->GetSockName());
@@ -484,6 +495,7 @@ void CModule::OnPreRehash() {}
 void CModule::OnPostRehash() {}
 void CModule::OnIRCDisconnected() {}
 void CModule::OnIRCConnected() {}
+CModule::EModRet CModule::OnIRCConnecting(CIRCSock *IRCSock) { return CONTINUE; }
 CModule::EModRet CModule::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { return CONTINUE; }
 CModule::EModRet CModule::OnBroadcast(CString& sMessage) { return CONTINUE; }
 
@@ -624,6 +636,7 @@ bool CModules::OnBoot() {
 bool CModules::OnPreRehash() { MODUNLOADCHK(OnPreRehash()); return false; }
 bool CModules::OnPostRehash() { MODUNLOADCHK(OnPostRehash()); return false; }
 bool CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); return false; }
+bool CModules::OnIRCConnecting(CIRCSock *pIRCSock) { MODHALTCHK(OnIRCConnecting(pIRCSock)); }
 bool CModules::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { MODHALTCHK(OnIRCRegistration(sPass, sNick, sIdent, sRealName)); }
 bool CModules::OnBroadcast(CString& sMessage) { MODHALTCHK(OnBroadcast(sMessage)); }
 bool CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); return false; }
@@ -715,79 +728,42 @@ CModule* CModules::FindModule(const CString& sModule) const {
 bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg, bool bFake) {
 	sRetMsg = "";
 
-	for (unsigned int a = 0; a < sModule.length(); a++) {
-		if (((sModule[a] < '0') || (sModule[a] > '9')) && ((sModule[a] < 'a') || (sModule[a] > 'z')) && ((sModule[a] < 'A') || (sModule[a] > 'Z')) && (sModule[a] != '_')) {
-			sRetMsg = "Unable to load module [" + sModule + "] module names can only be letters, numbers, or underscores.";
-			return false;
-		}
-	}
-
 	if (FindModule(sModule) != NULL) {
 		sRetMsg = "Module [" + sModule + "] already loaded.";
 		return false;
 	}
 
-	CString sModPath, sDataPath;
-
-	if (!CZNC::Get().FindModPath(sModule, sModPath, sDataPath)) {
-		sRetMsg = "Unable to find module [" + sModule + "]";
-		return false;
-	}
-
 	if (bFake) {
-		CModule* pModule = new CModule(NULL, sModule, sDataPath);
+		CModule* pModule = new CModule(NULL, sModule, "");
 		pModule->SetArgs(sArgs);
 		pModule->SetDescription("<<Fake Module>>");
 		pModule->SetFake(true);
 		push_back(pModule);
-		sRetMsg = "Loaded fake module [" + sModule + "] [" + sModPath + "]";
+		sRetMsg = "Loaded fake module [" + sModule + "]";
 		return true;
 	}
 
-	unsigned int uDLFlags = RTLD_NOW;
-	uDLFlags |= (pUser) ? RTLD_LOCAL : RTLD_GLOBAL;
+	CString sModPath, sDataPath;
+	CString sDesc;
+	bool bVersionMismatch;
+	bool bIsGlobal;
 
-	ModHandle p = dlopen((sModPath).c_str(), uDLFlags);
-
-	if (!p) {
-		sRetMsg = "Unable to load module [" + sModule + "] [" + dlerror() + "]";
+	if (!FindModPath(sModule, sModPath, sDataPath)) {
+		sRetMsg = "Unable to find module [" + sModule + "]";
 		return false;
 	}
 
-	typedef double (*dFP)();
-	dFP Version = (dFP) dlsym(p, "ZNCModVersion");
+	ModHandle p = OpenModule(sModule, sModPath, bVersionMismatch, bIsGlobal, sDesc, sRetMsg);
 
-	if (!Version) {
-		dlclose(p);
-		sRetMsg = "Could not find ZNCModVersion() in module [" + sModule + "]";
+	if (!p)
 		return false;
-	}
 
-	if (CModule::GetCoreVersion() != Version()) {
+	if (bVersionMismatch) {
 		dlclose(p);
 		sRetMsg = "Version mismatch, recompile this module.";
 		return false;
 	}
 
-	typedef bool (*bFP)();
-	bFP IsGlobal = (bFP) dlsym(p, "ZNCModGlobal");
-
-	if (!IsGlobal) {
-		dlclose(p);
-		sRetMsg = "Could not find ZNCModGlobal() in module [" + sModule + "]";
-		return false;
-	}
-
-	typedef CString (*sFP)();
-	sFP GetDesc = (sFP) dlsym(p, "ZNCModDescription");
-
-	if (!GetDesc) {
-		dlclose(p);
-		sRetMsg = "Could not find ZNCModDescription() in module [" + sModule + "]";
-		return false;
-	}
-
-	bool bIsGlobal = IsGlobal();
 	if ((pUser == NULL) != bIsGlobal) {
 		dlclose(p);
 		sRetMsg = "Module [" + sModule + "] is ";
@@ -824,7 +800,7 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 		pModule = Load(p, sModule, sDataPath);
 	}
 
-	pModule->SetDescription(GetDesc());
+	pModule->SetDescription(sDesc);
 	pModule->SetGlobal(bIsGlobal);
 	pModule->SetArgs(sArgs);
 	push_back(pModule);
@@ -927,59 +903,33 @@ bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser*
 	return true;
 }
 
-bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
-	for (unsigned int a = 0; a < sModule.length(); a++) {
-		const char& c = sModule[a];
-
-		if ((c < '0' || c > '9') && (c < 'a' || c > 'z') && (c < 'A' || c > 'Z') && c != '_') {
-			return false;
-		}
-	}
-
+bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule, CString& sRetMsg) {
 	CString sModPath, sTmp;
 
-	if (!CZNC::Get().FindModPath(sModule, sModPath, sTmp)) {
+	if (!FindModPath(sModule, sModPath, sTmp)) {
+		sRetMsg = "Unable to find module [" + sModule + "]";
 		return false;
 	}
 
-	unsigned int uDLFlags = RTLD_LAZY | RTLD_LOCAL;
+	return GetModPathInfo(ModInfo, sModule, sModPath, sRetMsg);
+}
 
-	ModHandle p = dlopen((sModPath).c_str(), uDLFlags);
+bool CModules::GetModPathInfo(CModInfo& ModInfo, const CString& sModule, const CString& sModPath, CString& sRetMsg) {
+	CString sDesc;
+	bool bVersionMismatch;
+	bool bIsGlobal;
 
-	if (!p) {
+	ModHandle p = OpenModule(sModule, sModPath, bVersionMismatch, bIsGlobal, sDesc, sRetMsg);
+
+	if (!p)
 		return false;
-	}
 
-	typedef double (*dFP)();
-	dFP Version = (dFP) dlsym(p, "ZNCModVersion");
-
-	if (!Version) {
-		dlclose(p);
-		return false;
-	}
-
-	typedef bool (*bFP)();
-	bFP IsGlobal = (bFP) dlsym(p, "ZNCModGlobal");
-
-	if (!IsGlobal) {
-		dlclose(p);
-		return false;
-	}
-
-	typedef CString (*sFP)();
-	sFP GetDescription = (sFP) dlsym(p, "ZNCModDescription");
-
-	if (!GetDescription) {
-		dlclose(p);
-		return false;
-	}
-
-	ModInfo.SetGlobal(IsGlobal());
-	ModInfo.SetDescription(GetDescription());
+	ModInfo.SetGlobal(bIsGlobal);
+	ModInfo.SetDescription(sDesc);
 	ModInfo.SetName(sModule);
 	ModInfo.SetPath(sModPath);
 
-	if (CModule::GetCoreVersion() != Version()) {
+	if (bVersionMismatch) {
 		ModInfo.SetDescription("--- Version mismatch, recompile this module. ---");
 	}
 
@@ -994,47 +944,133 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 	unsigned int a = 0;
 	CDir Dir;
 
-	Dir.FillByWildcard(CZNC::Get().GetCurPath() + "/modules", "*.so");
-	for (a = 0; a < Dir.size(); a++) {
-		CFile& File = *Dir[a];
-		CString sName = File.GetShortName();
-		CModInfo ModInfo;
-		sName.RightChomp(3);
+	ModDirList dirs = GetModDirs();
 
-		if (GetModInfo(ModInfo, sName)) {
-			if (ModInfo.IsGlobal() == bGlobal) {
-				ssMods.insert(ModInfo);
-			}
-		}
-	}
+	while (!dirs.empty()) {
+		Dir.FillByWildcard(dirs.front().first, "*.so");
+		dirs.pop();
 
-	Dir.FillByWildcard(CZNC::Get().GetModPath(), "*.so");
-	for (a = 0; a < Dir.size(); a++) {
-		CFile& File = *Dir[a];
-		CString sName = File.GetShortName();
-		CModInfo ModInfo;
-		sName.RightChomp(3);
+		for (a = 0; a < Dir.size(); a++) {
+			CFile& File = *Dir[a];
+			CString sName = File.GetShortName();
+			CString sPath = File.GetLongName();
+			CModInfo ModInfo;
+			sName.RightChomp(3);
 
-		if (GetModInfo(ModInfo, sName)) {
-			if (ModInfo.IsGlobal() == bGlobal) {
-				ssMods.insert(ModInfo);
-			}
-		}
-	}
-
-	Dir.FillByWildcard(_MODDIR_, "*.so");
-	for (a = 0; a < Dir.size(); a++) {
-		CFile& File = *Dir[a];
-		CString sName = File.GetShortName();
-		CModInfo ModInfo;
-		sName.RightChomp(3);
-
-		if (GetModInfo(ModInfo, sName)) {
-			if (ModInfo.IsGlobal() == bGlobal) {
-				ssMods.insert(ModInfo);
+			CString sIgnoreRetMsg;
+			if (GetModPathInfo(ModInfo, sName, sPath, sIgnoreRetMsg)) {
+				if (ModInfo.IsGlobal() == bGlobal) {
+					ssMods.insert(ModInfo);
+				}
 			}
 		}
 	}
 }
 
+bool CModules::FindModPath(const CString& sModule, CString& sModPath,
+		CString& sDataPath) {
+	CString sMod = sModule;
+	CString sDir = sMod;
+	if (sModule.find(".") == CString::npos)
+		sMod += ".so";
+
+	ModDirList dirs = GetModDirs();
+
+	while (!dirs.empty()) {
+		sModPath = dirs.front().first + sMod;
+		sDataPath = dirs.front().second;
+		dirs.pop();
+
+		if (CFile::Exists(sModPath)) {
+			sDataPath += sDir;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+CModules::ModDirList CModules::GetModDirs() {
+	ModDirList ret;
+
+	// ./modules
+	CString sDir = CZNC::Get().GetCurPath() + "/modules/";
+	ret.push(std::make_pair(sDir, sDir));
+
+	// ./modules/extra
+	sDir = CZNC::Get().GetCurPath() + "/modules/extra/";
+	ret.push(std::make_pair(sDir, sDir));
+
+	// ~/.znc/modules
+	sDir = CZNC::Get().GetModPath() + "/";
+	ret.push(std::make_pair(sDir, sDir));
+
+	// <moduledir> and <datadir> (<prefix>/lib/znc)
+	ret.push(std::make_pair(_MODDIR_ + CString("/"), _DATADIR_ + CString("/")));
+
+	return ret;
+}
+
+ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath, bool &bVersionMismatch,
+		bool &bIsGlobal, CString& sDesc, CString& sRetMsg) {
+	// Some sane defaults in case anything errors out below
+	bVersionMismatch = false;
+	bIsGlobal = false;
+	sDesc.clear();
+	sRetMsg.clear();
+
+	for (unsigned int a = 0; a < sModule.length(); a++) {
+		if (((sModule[a] < '0') || (sModule[a] > '9')) && ((sModule[a] < 'a') || (sModule[a] > 'z')) && ((sModule[a] < 'A') || (sModule[a] > 'Z')) && (sModule[a] != '_')) {
+			sRetMsg = "Module names can only contain letters, numbers and underscores, [" + sModule + "] is invalid.";
+			return NULL;
+		}
+	}
+
+	ModHandle p = dlopen((sModPath).c_str(), RTLD_NOW | RTLD_LOCAL);
+
+	if (!p) {
+		sRetMsg = "Unable to open module [" + sModule + "] [" + dlerror() + "]";
+		return NULL;
+	}
+
+	typedef double (*dFP)();
+	dFP Version = (dFP) dlsym(p, "ZNCModVersion");
+
+	if (!Version) {
+		dlclose(p);
+		sRetMsg = "Could not find ZNCModVersion() in module [" + sModule + "]";
+		return NULL;
+	}
+
+	typedef bool (*bFP)();
+	bFP IsGlobal = (bFP) dlsym(p, "ZNCModGlobal");
+
+	if (!IsGlobal) {
+		dlclose(p);
+		sRetMsg = "Could not find ZNCModGlobal() in module [" + sModule + "]";
+		return NULL;
+	}
+
+	typedef const char *(*sFP)();
+	sFP GetDesc = (sFP) dlsym(p, "ZNCModDescription");
+
+	if (!GetDesc) {
+		dlclose(p);
+		sRetMsg = "Could not find ZNCModDescription() in module [" + sModule + "]";
+		return false;
+	}
+
+	if (CModule::GetCoreVersion() != Version()) {
+		bVersionMismatch = true;
+		sRetMsg = "Version mismatch, recompile this module.";
+	} else {
+		sRetMsg = "";
+		bVersionMismatch = false;
+		bIsGlobal = IsGlobal();
+		sDesc = GetDesc();
+	}
+
+	return p;
+}
+
 #endif // _MODULES
