@@ -14,15 +14,15 @@
 #include "FileUtils.h"
 #include "Utils.h"
 #include <set>
-#include <vector>
+#include <queue>
 
-using std::vector;
 using std::set;
 
 // Forward Declarations
 class CAuthBase;
 class CChan;
 class CClient;
+class CIRCSock;
 // !Forward Declarations
 
 // User Module Macros
@@ -38,22 +38,25 @@ class CClient;
 
 typedef void* ModHandle;
 
+#define MODCOMMONDEFS(DESCRIPTION, GLOBAL) \
+	const char *ZNCModDescription(); \
+	bool ZNCModGlobal(); \
+	double ZNCModVersion(); \
+	const char *ZNCModDescription() { return DESCRIPTION; } \
+	double ZNCModVersion() { return VERSION; } \
+	bool ZNCModGlobal() { return GLOBAL; } \
+
 #define MODCONSTRUCTOR(CLASS) \
 	CLASS(ModHandle pDLL, CUser* pUser, const CString& sModName, \
 			const CString& sModPath) \
 			: CModule(pDLL, pUser, sModName, sModPath)
 #define MODULEDEFS(CLASS, DESCRIPTION) \
 	extern "C" { \
+		MODCOMMONDEFS(DESCRIPTION, false) \
 		/* First the definitions to shut up some compiler warnings */ \
-		CString ZNCModDescription(); \
-		bool ZNCModGlobal(); \
-		double ZNCModVersion(); \
 		CModule* ZNCModLoad(ModHandle p, CUser* pUser, const CString& sModName, \
 				const CString& sModPath); \
 		void ZNCModUnload(CModule* pMod); \
-		CString ZNCModDescription() { return DESCRIPTION; } \
-		bool ZNCModGlobal() { return false; } \
-		double ZNCModVersion() { return VERSION; } \
 		CModule* ZNCModLoad(ModHandle p, CUser* pUser, const CString& sModName, \
 				const CString& sModPath) \
 		{ return new CLASS(p, pUser, sModName, sModPath); } \
@@ -67,16 +70,11 @@ typedef void* ModHandle;
 			: CGlobalModule(pDLL, sModName, sModPath)
 #define GLOBALMODULEDEFS(CLASS, DESCRIPTION) \
 	extern "C" { \
+		MODCOMMONDEFS(DESCRIPTION, true) \
 		/* First the definitions to shut up some compiler warnings */ \
-		CString ZNCModDescription(); \
-		bool ZNCModGlobal(); \
-		double ZNCModVersion(); \
 		CGlobalModule* ZNCModLoad(ModHandle p, const CString& sModName, \
 				const CString& sModPath); \
 		void ZNCModUnload(CGlobalModule* pMod); \
-		CString ZNCModDescription() { return DESCRIPTION; } \
-		bool ZNCModGlobal() { return true; } \
-		double ZNCModVersion() { return VERSION; } \
 		CGlobalModule* ZNCModLoad(ModHandle p, const CString& sModName, \
 				const CString& sModPath) \
 		{ return new CLASS(p, sModName, sModPath); } \
@@ -238,6 +236,7 @@ public:
 	virtual void OnPostRehash();
 	virtual void OnIRCDisconnected();
 	virtual void OnIRCConnected();
+	virtual EModRet OnIRCConnecting(CIRCSock *pIRCSock);
 	virtual EModRet OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
 	virtual EModRet OnBroadcast(CString& sMessage);
 
@@ -314,11 +313,12 @@ public:
 	// Timer stuff
 	bool AddTimer(CTimer* pTimer);
 	bool AddTimer(FPTimer_t pFBCallback, const CString& sLabel, u_int uInterval, u_int uCycles = 0, const CString& sDescription = "");
+	bool RemTimer(CTimer* pTimer);
 	bool RemTimer(const CString& sLabel);
 	bool UnlinkTimer(CTimer* pTimer);
 	CTimer* FindTimer(const CString& sLabel);
-	vector<CTimer*>::const_iterator BeginTimers() const { return m_vTimers.begin(); }
-	vector<CTimer*>::const_iterator EndTimers() const { return m_vTimers.end(); }
+	set<CTimer*>::const_iterator BeginTimers() const { return m_sTimers.begin(); }
+	set<CTimer*>::const_iterator EndTimers() const { return m_sTimers.end(); }
 	virtual void ListTimers();
 	// !Timer stuff
 
@@ -328,8 +328,8 @@ public:
 	bool RemSocket(const CString& sSockName);
 	bool UnlinkSocket(CSocket* pSocket);
 	CSocket* FindSocket(const CString& sSockName);
-	vector<CSocket*>::const_iterator BeginSockets() const { return m_vSockets.begin(); }
-	vector<CSocket*>::const_iterator EndSockets() const { return m_vSockets.end(); }
+	set<CSocket*>::const_iterator BeginSockets() const { return m_sSockets.begin(); }
+	set<CSocket*>::const_iterator EndSockets() const { return m_sSockets.end(); }
 	virtual void ListSockets();
 	// !Socket stuff
 
@@ -367,8 +367,8 @@ protected:
 	bool			m_bFake;
 	bool			m_bGlobal;
 	CString			m_sDescription;
-	vector<CTimer*>		m_vTimers;
-	vector<CSocket*>	m_vSockets;
+	set<CTimer*>		m_sTimers;
+	set<CSocket*>		m_sSockets;
 	ModHandle 		m_pDLL;
 	CSockManager*		m_pManager;
 	CUser*			m_pUser;
@@ -398,6 +398,7 @@ public:
 	bool OnPostRehash();
 	bool OnIRCDisconnected();
 	bool OnIRCConnected();
+	bool OnIRCConnecting(CIRCSock *pIRCSock);
 	bool OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
 	bool OnBroadcast(CString& sMessage);
 
@@ -460,9 +461,23 @@ public:
 	bool UnloadModule(const CString& sModule, CString& sRetMsg);
 	bool ReloadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg);
 
-	bool GetModInfo(CModInfo& ModInfo, const CString& sModule);
+	bool GetModInfo(CModInfo& ModInfo, const CString& sModule, CString &sRetMsg);
+	bool GetModPathInfo(CModInfo& ModInfo, const CString& sModule, const CString& sModPath, CString &sRetMsg);
 	void GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal = false);
 
+	// This returns the path to the .so and to the data dir
+	// which is where static data (webadmin skins) are saved
+	static bool FindModPath(const CString& sModule, CString& sModPath,
+			CString& sDataPath);
+	// Return a list of <module dir, data dir> pairs for directories in
+	// which modules can be found.
+	typedef std::queue<std::pair<CString, CString> > ModDirList;
+	static ModDirList GetModDirs();
+
+private:
+	ModHandle OpenModule(const CString& sModule, const CString& sModPath,
+			bool &bVersionMismatch, bool &bIsGlobal, CString& sDesc, CString& sRetMsg);
+
 protected:
 	CUser*		m_pUser;
 	CClient*	m_pClient;
