@@ -32,10 +32,11 @@ CUser::CUser(const CString& sUserName) {
 	m_MotdBuffer.SetLineCount(200);		// This should be more than enough motd lines
 	m_bMultiClients = true;
 	m_bBounceDCCs = true;
-	m_bPassHashed = false;
+	m_eHashType = HASH_NONE;
 	m_bUseClientIP = false;
 	m_bDenyLoadMod = false;
 	m_bAdmin= false;
+	m_bIRCAway = false;
 	m_bDenySetVHost= false;
 	m_sStatusPrefix = "*";
 	m_sChanPrefixes = "";
@@ -48,10 +49,8 @@ CUser::CUser(const CString& sUserName) {
 	m_bAppendTimestamp = false;
 	m_bPrependTimestamp = true;
 	m_bIRCConnectEnabled = true;
-	m_pJoinTimer = new CJoinTimer(this);
-	m_pMiscTimer = new CMiscTimer(this);
-	CZNC::Get().GetManager().AddCron(m_pJoinTimer);
-	CZNC::Get().GetManager().AddCron(m_pMiscTimer);
+	m_pUserTimer = new CUserTimer(this);
+	CZNC::Get().GetManager().AddCron(m_pUserTimer);
 	m_sUserPath = CZNC::Get().GetUserPath() + "/" + sUserName;
 	m_sDLPath = GetUserPath() + "/downloads";
 }
@@ -76,8 +75,7 @@ CUser::~CUser() {
 	while (!m_sDCCSocks.empty())
 		CZNC::Get().GetManager().DelSockByAddr((CZNCSock*) *m_sDCCSocks.begin());
 
-	CZNC::Get().GetManager().DelCronByAddr(m_pJoinTimer);
-	CZNC::Get().GetManager().DelCronByAddr(m_pMiscTimer);
+	CZNC::Get().GetManager().DelCronByAddr(m_pUserTimer);
 }
 
 #ifdef _MODULES
@@ -150,6 +148,7 @@ void CUser::IRCDisconnected() {
 	}
 
 	SetIRCServer("");
+	m_bIRCAway = false;
 
 	// Get the reconnect going
 	CheckIRCConnect();
@@ -199,17 +198,16 @@ CString CUser::AddTimestamp(const CString& sStr) const {
 CString& CUser::AddTimestamp(const CString& sStr, CString& sRet) const {
 	char szTimestamp[1024];
 	time_t tm;
+	sRet = sStr;
 
-	if (GetTimestampFormat().empty() || (!m_bAppendTimestamp && !m_bPrependTimestamp)) {
-		sRet = sStr;
-	} else {
+	if (!GetTimestampFormat().empty() && (m_bAppendTimestamp || m_bPrependTimestamp)) {
 		time(&tm);
 		tm += (time_t)(m_fTimezoneOffset * 60 * 60); // offset is in hours
 		size_t i = strftime(szTimestamp, sizeof(szTimestamp), GetTimestampFormat().c_str(), localtime(&tm));
-		if (i != 0) {
-			sRet = sStr;
-		} else {
-			sRet.clear();
+		// If strftime returns 0, an error occured in format, or result is empty
+		// In both cases just don't prepend/append anything to our string
+		if (0 == i) {
+			return sRet;
 		}
 
 		if (m_bPrependTimestamp) {
@@ -274,10 +272,16 @@ void CUser::UserConnected(CClient* pClient) {
 			sUserMode += *it;
 		}
 		if (!sUserMode.empty()) {
-			pClient->PutClient(":" + GetIRCNick().GetNick() + " MODE " + GetIRCNick().GetNick() + " :+" + sUserMode);
+			pClient->PutClient(":" + GetIRCNick().GetNickMask() + " MODE " + GetIRCNick().GetNick() + " :+" + sUserMode);
 		}
 	}
 
+	if (m_bIRCAway) {
+		// If they want to know their away reason they'll have to whois
+		// themselves. At least we can tell them their away status...
+		pClient->PutClient(":irc.znc.in 306 " + GetIRCNick().GetNick() + " :You have been marked as being away");
+	}
+
 	const vector<CChan*>& vChans = GetChans();
 	for (unsigned int a = 0; a < vChans.size(); a++) {
 		if ((vChans[a]->IsOn()) && (!vChans[a]->IsDetached())) {
@@ -324,7 +328,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	}
 
 	if (!User.GetPass().empty()) {
-		SetPass(User.GetPass(), User.IsPassHashed(), User.GetPassSalt());
+		SetPass(User.GetPass(), User.GetPassHashType(), User.GetPassSalt());
 	}
 
 	SetNick(User.GetNick(false));
@@ -333,6 +337,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetRealName(User.GetRealName());
 	SetStatusPrefix(User.GetStatusPrefix());
 	SetVHost(User.GetVHost());
+	SetDCCVHost(User.GetDCCVHost());
 	SetQuitMsg(User.GetQuitMsg());
 	SetDefaultChanModes(User.GetDefaultChanModes());
 	SetBufferCount(User.GetBufferCount());
@@ -357,39 +362,6 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 
 	// !Allowed Hosts
 
-#ifdef _MODULES
-	// Modules
-	set<CString> ssUnloadMods;
-	CModules& vCurMods = GetModules();
-	const CModules& vNewMods = User.GetModules();
-
-	for (a = 0; a < vNewMods.size(); a++) {
-		CString sModRet;
-		CModule* pNewMod = vNewMods[a];
-		CModule* pCurMod = vCurMods.FindModule(pNewMod->GetModName());
-
-		if (!pCurMod) {
-			vCurMods.LoadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
-		} else if (pNewMod->GetArgs() != pCurMod->GetArgs()) {
-			vCurMods.ReloadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
-		}
-	}
-
-	for (a = 0; a < vCurMods.size(); a++) {
-		CModule* pCurMod = vCurMods[a];
-		CModule* pNewMod = vNewMods.FindModule(pCurMod->GetModName());
-
-		if (!pNewMod) {
-			ssUnloadMods.insert(pCurMod->GetModName());
-		}
-	}
-
-	for (set<CString>::iterator it = ssUnloadMods.begin(); it != ssUnloadMods.end(); it++) {
-		vCurMods.UnloadModule(*it);
-	}
-	// !Modules
-#endif // !_MODULES
-
 	// Servers
 	const vector<CServer*>& vServers = User.GetServers();
 	CString sServer;
@@ -406,20 +378,20 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 		AddServer(pServer->GetName(), pServer->GetPort(), pServer->GetPass(), pServer->IsSSL());
 	}
 
+	m_uServerIdx = 0;
 	for (a = 0; a < m_vServers.size(); a++) {
 		if (sServer.Equals(m_vServers[a]->GetName())) {
-			m_uServerIdx = a +1;
+			m_uServerIdx = a + 1;
 			break;
 		}
+	}
+	if (m_uServerIdx == 0) {
+		m_uServerIdx = m_vServers.size();
+		CIRCSock* pSock = GetIRCSock();
 
-		if (a == m_vServers.size() -1) {
-			m_uServerIdx = m_vServers.size();
-			CIRCSock* pSock = GetIRCSock();
-
-			if (pSock) {
-				PutStatus("Jumping servers because this server is no longer in the list");
-				pSock->Quit();
-			}
+		if (pSock) {
+			PutStatus("Jumping servers because this server is no longer in the list");
+			pSock->Quit();
 		}
 	}
 	// !Servers
@@ -472,6 +444,39 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetTimezoneOffset(User.GetTimezoneOffset());
 	// !Flags
 
+#ifdef _MODULES
+	// Modules
+	set<CString> ssUnloadMods;
+	CModules& vCurMods = GetModules();
+	const CModules& vNewMods = User.GetModules();
+
+	for (a = 0; a < vNewMods.size(); a++) {
+		CString sModRet;
+		CModule* pNewMod = vNewMods[a];
+		CModule* pCurMod = vCurMods.FindModule(pNewMod->GetModName());
+
+		if (!pCurMod) {
+			vCurMods.LoadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
+		} else if (pNewMod->GetArgs() != pCurMod->GetArgs()) {
+			vCurMods.ReloadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
+		}
+	}
+
+	for (a = 0; a < vCurMods.size(); a++) {
+		CModule* pCurMod = vCurMods[a];
+		CModule* pNewMod = vNewMods.FindModule(pCurMod->GetModName());
+
+		if (!pNewMod) {
+			ssUnloadMods.insert(pCurMod->GetModName());
+		}
+	}
+
+	for (set<CString>::iterator it = ssUnloadMods.begin(); it != ssUnloadMods.end(); it++) {
+		vCurMods.UnloadModule(*it);
+	}
+	// !Modules
+#endif // !_MODULES
+
 	return true;
 }
 
@@ -600,11 +605,14 @@ bool CUser::PrintLine(CFile& File, const CString& sName, const CString& sValue)
 bool CUser::WriteConfig(CFile& File) {
 	File.Write("<User " + GetUserName().FirstLine() + ">\n");
 
-	if (IsPassHashed()) {
+	if (m_eHashType != HASH_NONE) {
+		CString sHash = "md5";
+		if (m_eHashType == HASH_SHA256)
+			sHash = "sha256";
 		if (m_sPassSalt.empty()) {
-			PrintLine(File, "Pass", "md5#" + GetPass());
+			PrintLine(File, "Pass", sHash + "#" + GetPass());
 		} else {
-			PrintLine(File, "Pass", "md5#" + GetPass() + "#" + m_sPassSalt + "#");
+			PrintLine(File, "Pass", sHash + "#" + GetPass() + "#" + m_sPassSalt + "#");
 		}
 	} else {
 		PrintLine(File, "Pass", "plain#" + GetPass());
@@ -614,6 +622,7 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "Ident", GetIdent());
 	PrintLine(File, "RealName", GetRealName());
 	PrintLine(File, "VHost", GetVHost());
+	PrintLine(File, "DCCVHost", GetDCCVHost());
 	PrintLine(File, "QuitMsg", GetQuitMsg());
 	if (CZNC::Get().GetStatusPrefix() != GetStatusPrefix())
 		PrintLine(File, "StatusPrefix", GetStatusPrefix());
@@ -704,27 +713,43 @@ CChan* CUser::FindChan(const CString& sName) const {
 }
 
 void CUser::JoinChans() {
+	// Avoid divsion by zero, it's bad!
+	if (m_vChans.size() == 0)
+		return;
+
+	// We start at a random offset into the channel list so that if your
+	// first 3 channels are invite-only and you got MaxJoins == 3, ZNC will
+	// still be able to join the rest of your channels.
+	unsigned int start = rand() % m_vChans.size();
 	unsigned int uJoins = m_uMaxJoins;
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
-		CChan* pChan = m_vChans[a];
+		unsigned int idx = (start + a) % m_vChans.size();
+		CChan* pChan = m_vChans[idx];
 		if (!pChan->IsOn() && !pChan->IsDisabled()) {
-			if (JoinTries() != 0 && pChan->GetJoinTries() >= JoinTries()) {
-				PutStatus("The channel " + pChan->GetName() + " could not be joined, disabling it.");
-				pChan->Disable();
-			} else {
-				pChan->IncJoinTries();
-				MODULECALL(OnTimerAutoJoin(*pChan), this, NULL, continue);
+			if (!JoinChan(pChan))
+				continue;
 
-				PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
-
-				// Limit the number of joins
-				if (uJoins != 0 && --uJoins == 0)
-					return;
-			}
+			// Limit the number of joins
+			if (uJoins != 0 && --uJoins == 0)
+				return;
 		}
 	}
 }
 
+bool CUser::JoinChan(CChan* pChan) {
+	if (JoinTries() != 0 && pChan->GetJoinTries() >= JoinTries()) {
+		PutStatus("The channel " + pChan->GetName() + " could not be joined, disabling it.");
+		pChan->Disable();
+	} else {
+		pChan->IncJoinTries();
+		MODULECALL(OnTimerAutoJoin(*pChan), this, NULL, return false);
+
+		PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
+		return true;
+	}
+	return false;
+}
+
 CServer* CUser::FindServer(const CString& sName) const {
 	for (unsigned int a = 0; a < m_vServers.size(); a++) {
 		CServer* pServer = m_vServers[a];
@@ -736,39 +761,54 @@ CServer* CUser::FindServer(const CString& sName) const {
 	return NULL;
 }
 
-bool CUser::DelServer(const CString& sName) {
+bool CUser::DelServer(const CString& sName, unsigned short uPort, const CString& sPass) {
 	if (sName.empty()) {
 		return false;
 	}
 
 	unsigned int a = 0;
+	bool bSawCurrentServer = false;
+	CServer* pCurServer = GetCurrentServer();
 
 	for (vector<CServer*>::iterator it = m_vServers.begin(); it != m_vServers.end(); it++, a++) {
 		CServer* pServer = *it;
 
-		if (pServer->GetName().Equals(sName)) {
-			CServer* pCurServer = GetCurrentServer();
-			m_vServers.erase(it);
+		if (pServer == pCurServer)
+			bSawCurrentServer = true;
 
-			if (pServer == pCurServer) {
-				CIRCSock* pIRCSock = GetIRCSock();
+		if (!pServer->GetName().Equals(sName))
+			continue;
 
-				if (m_uServerIdx) {
-					m_uServerIdx--;
-				}
+		if (uPort != 0 && pServer->GetPort() != uPort)
+			continue;
 
-				if (pIRCSock) {
-					pIRCSock->Quit();
-					PutStatus("Your current server was removed, jumping...");
-				}
-			} else if (m_uServerIdx >= m_vServers.size()) {
-				m_uServerIdx = 0;
+		if (!sPass.empty() && pServer->GetPass() != sPass)
+			continue;
+
+		m_vServers.erase(it);
+
+		if (pServer == pCurServer) {
+			CIRCSock* pIRCSock = GetIRCSock();
+
+			// Make sure we don't skip the next server in the list!
+			if (m_uServerIdx) {
+				m_uServerIdx--;
 			}
 
-			delete pServer;
-
-			return true;
+			if (pIRCSock) {
+				pIRCSock->Quit();
+				PutStatus("Your current server was removed, jumping...");
+			}
+		} else if (!bSawCurrentServer) {
+			// Our current server comes after the server which we
+			// are removing. This means that it now got a different
+			// index in m_vServers!
+			m_uServerIdx--;
 		}
+
+		delete pServer;
+
+		return true;
 	}
 
 	return false;
@@ -867,11 +907,16 @@ CServer* CUser::GetCurrentServer() const {
 }
 
 bool CUser::CheckPass(const CString& sPass) const {
-	if (!m_bPassHashed) {
+	switch (m_eHashType)
+	{
+	case HASH_MD5:
+		return m_sPass.Equals(CUtils::SaltedMD5Hash(sPass, m_sPassSalt));
+	case HASH_SHA256:
+		return m_sPass.Equals(CUtils::SaltedSHA256Hash(sPass, m_sPassSalt));
+	case HASH_NONE:
+	default:
 		return (sPass == m_sPass);
 	}
-
-	return m_sPass.Equals(CUtils::SaltedHash(sPass, m_sPassSalt));
 }
 
 /*CClient* CUser::GetClient() {
@@ -905,6 +950,12 @@ CString CUser::GetLocalIP() {
 	return "";
 }
 
+CString CUser::GetLocalDCCIP() {
+	if (!GetDCCVHost().empty())
+		return GetDCCVHost();
+	return GetLocalIP();
+}
+
 bool CUser::PutIRC(const CString& sLine) {
 	CIRCSock* pIRCSock = GetIRCSock();
 
@@ -1004,13 +1055,13 @@ bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const
 		return false;
 	}
 
-	unsigned short uPort = CZNC::Get().GetManager().ListenRand("DCC::LISTEN::" + sRemoteNick, GetLocalIP(), false, SOMAXCONN, pSock, 120);
+	unsigned short uPort = CZNC::Get().GetManager().ListenRand("DCC::LISTEN::" + sRemoteNick, GetLocalDCCIP(), false, SOMAXCONN, pSock, 120);
 
 	if (GetNick().Equals(sRemoteNick)) {
-		PutUser(":" + GetStatusPrefix() + "status!znc@znc.in PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
+		PutUser(":" + GetStatusPrefix() + "status!znc@znc.in PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalDCCIP())) + " "
 				+ CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	} else {
-		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
+		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalDCCIP())) + " "
 			    + CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	}
 
@@ -1031,7 +1082,7 @@ bool CUser::GetFile(const CString& sRemoteNick, const CString& sRemoteIP, unsign
 		return false;
 	}
 
-	if (!CZNC::Get().GetManager().Connect(sRemoteIP, uRemotePort, "DCC::GET::" + sRemoteNick, 60, false, GetLocalIP(), pSock)) {
+	if (!CZNC::Get().GetManager().Connect(sRemoteIP, uRemotePort, "DCC::GET::" + sRemoteNick, 60, false, GetLocalDCCIP(), pSock)) {
 		PutModule(sModuleName, "DCC <- [" + sRemoteNick + "][" + sFileName + "] - Unable to connect.");
 		return false;
 	}
@@ -1078,9 +1129,10 @@ void CUser::SetAltNick(const CString& s) { m_sAltNick = s; }
 void CUser::SetIdent(const CString& s) { m_sIdent = s; }
 void CUser::SetRealName(const CString& s) { m_sRealName = s; }
 void CUser::SetVHost(const CString& s) { m_sVHost = s; }
-void CUser::SetPass(const CString& s, bool bHashed, const CString& sSalt) {
+void CUser::SetDCCVHost(const CString& s) { m_sDCCVHost = s; }
+void CUser::SetPass(const CString& s, eHashType eHash, const CString& sSalt) {
 	m_sPass = s;
-	m_bPassHashed = bHashed;
+	m_eHashType = eHash;
 	m_sPassSalt = sSalt;
 }
 void CUser::SetMultiClients(bool b) { m_bMultiClients = b; }
@@ -1137,8 +1189,9 @@ const CString& CUser::GetAltNick(bool bAllowDefault) const { return (bAllowDefau
 const CString& CUser::GetIdent(bool bAllowDefault) const { return (bAllowDefault && m_sIdent.empty()) ? GetCleanUserName() : m_sIdent; }
 const CString& CUser::GetRealName() const { return m_sRealName.empty() ? m_sUserName : m_sRealName; }
 const CString& CUser::GetVHost() const { return m_sVHost; }
+const CString& CUser::GetDCCVHost() const { return m_sDCCVHost; }
 const CString& CUser::GetPass() const { return m_sPass; }
-bool CUser::IsPassHashed() const { return m_bPassHashed; }
+CUser::eHashType CUser::GetPassHashType() const { return m_eHashType; }
 const CString& CUser::GetPassSalt() const { return m_sPassSalt; }
 
 bool CUser::ConnectPaused() {
@@ -1167,7 +1220,7 @@ const vector<CChan*>& CUser::GetChans() const { return m_vChans; }
 const vector<CServer*>& CUser::GetServers() const { return m_vServers; }
 const CNick& CUser::GetIRCNick() const { return m_IRCNick; }
 const CString& CUser::GetIRCServer() const { return m_sIRCServer; }
-CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.empty()) ? m_sQuitMsg : CZNC::GetTag(false); }
+CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.Trim_n().empty()) ? m_sQuitMsg : CZNC::GetTag(false); }
 const MCString& CUser::GetCTCPReplies() const { return m_mssCTCPReplies; }
 unsigned int CUser::GetBufferCount() const { return m_uBufferCount; }
 bool CUser::KeepBuffer() const { return m_bKeepBuffer; }
