@@ -24,8 +24,7 @@ using std::vector;
 class CChan;
 class CClient;
 class CIRCSock;
-class CJoinTimer;
-class CMiscTimer;
+class CUserTimer;
 class CServer;
 class CDCCBounce;
 class CDCCSock;
@@ -35,6 +34,20 @@ public:
 	CUser(const CString& sUserName);
 	~CUser();
 
+	enum eHashType {
+		HASH_NONE,
+		HASH_MD5,
+		HASH_SHA256,
+
+		HASH_DEFAULT = HASH_SHA256
+	};
+
+	// If you change the default hash here and in HASH_DEFAULT,
+	// don't forget CUtils::sDefaultHash!
+	static CString SaltedHash(const CString& sPass, const CString& sSalt) {
+		return CUtils::SaltedSHA256Hash(sPass, sSalt);
+	}
+
 	bool PrintLine(CFile& File, const CString& sName, const CString& sValue);
 	bool WriteConfig(CFile& File);
 	CChan* FindChan(const CString& sName) const;
@@ -42,8 +55,9 @@ public:
 	bool AddChan(const CString& sName, bool bInConfig);
 	bool DelChan(const CString& sName);
 	void JoinChans();
+	bool JoinChan(CChan* pChan);
 	CServer* FindServer(const CString& sName) const;
-	bool DelServer(const CString& sName);
+	bool DelServer(const CString& sName, unsigned short uPort, const CString& sPass);
 	bool AddServer(const CString& sName);
 	bool AddServer(const CString& sName, unsigned short uPort, const CString& sPass = "", bool bSSL = false);
 	CServer* GetNextServer();
@@ -73,13 +87,16 @@ public:
 
 	// Buffers
 	void AddRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.AddLine(sPre, sPost, bIncNick); }
-	void AddMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.AddLine(sPre, sPost, bIncNick); }
-	void AddQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.AddLine(sPre, sPost, bIncNick); }
 	void UpdateRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.UpdateLine(sPre, sPost, bIncNick); }
-	void UpdateMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.UpdateLine(sPre, sPost, bIncNick); }
-	void UpdateQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.UpdateLine(sPre, sPost, bIncNick); }
+	void UpdateExactRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.UpdateExactLine(sPre, sPost, bIncNick); }
 	void ClearRawBuffer() { m_RawBuffer.Clear(); }
+
+	void AddMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.AddLine(sPre, sPost, bIncNick); }
+	void UpdateMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.UpdateLine(sPre, sPost, bIncNick); }
 	void ClearMotdBuffer() { m_MotdBuffer.Clear(); }
+
+	void AddQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.AddLine(sPre, sPost, bIncNick); }
+	void UpdateQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.UpdateLine(sPre, sPost, bIncNick); }
 	void ClearQueryBuffer() { m_QueryBuffer.Clear(); }
 	// !Buffers
 
@@ -94,6 +111,7 @@ public:
 	void UserDisconnected(CClient* pClient);
 
 	CString GetLocalIP();
+	CString GetLocalDCCIP();
 	bool IsIRCConnected() const { return GetIRCSock() != NULL; }
 	void IRCConnected(CIRCSock* pIRCSock);
 	void IRCDisconnected();
@@ -127,7 +145,8 @@ public:
 	void SetIdent(const CString& s);
 	void SetRealName(const CString& s);
 	void SetVHost(const CString& s);
-	void SetPass(const CString& s, bool bHashed, const CString& sSalt = "");
+	void SetDCCVHost(const CString& s);
+	void SetPass(const CString& s, eHashType eHash, const CString& sSalt = "");
 	void SetBounceDCCs(bool b);
 	void SetMultiClients(bool b);
 	void SetUseClientIP(bool b);
@@ -151,6 +170,7 @@ public:
 	void SetJoinTries(unsigned int i) { m_uMaxJoinTries = i; }
 	void SetMaxJoins(unsigned int i) { m_uMaxJoins = i; }
 	void SetIRCConnectEnabled(bool b) { m_bIRCConnectEnabled = b; }
+	void SetIRCAway(bool b) { m_bIRCAway = b; }
 	// !Setters
 
 	// Getters
@@ -164,8 +184,9 @@ public:
 	const CString& GetIdent(bool bAllowDefault = true) const;
 	const CString& GetRealName() const;
 	const CString& GetVHost() const;
+	const CString& GetDCCVHost() const;
 	const CString& GetPass() const;
-	bool IsPassHashed() const;
+	eHashType GetPassHashType() const;
 	const CString& GetPassSalt() const;
 	const set<CString>& GetAllowedHosts() const;
 	const CString& GetTimestampFormat() const;
@@ -196,7 +217,7 @@ public:
 	unsigned int GetBufferCount() const;
 	bool KeepBuffer() const;
 	bool IsBeingDeleted() const { return m_bBeingDeleted; }
-	bool HasServers() const { return m_vServers.size() > 0; }
+	bool HasServers() const { return !m_vServers.empty(); }
 	float GetTimezoneOffset() const { return m_fTimezoneOffset; }
 	unsigned long long BytesRead() const { return m_uBytesRead; }
 	unsigned long long BytesWritten() const { return m_uBytesWritten; }
@@ -213,17 +234,20 @@ protected:
 	CString			m_sIdent;
 	CString			m_sRealName;
 	CString			m_sVHost;
+	CString			m_sDCCVHost;
 	CString			m_sPass;
 	CString			m_sPassSalt;
 	CString			m_sStatusPrefix;
 	CString			m_sDefaultChanModes;
 	CString			m_sChanPrefixes;
 	CNick			m_IRCNick;
+	bool			m_bIRCAway;
 	CString			m_sIRCServer;
 	CString			m_sQuitMsg;
 	MCString		m_mssCTCPReplies;
 	CString			m_sTimestampFormat;
 	float			m_fTimezoneOffset;
+	eHashType		m_eHashType;
 
 	// Paths
 	CString			m_sUserPath;
@@ -235,7 +259,6 @@ protected:
 	CBuffer				m_QueryBuffer;
 	bool				m_bMultiClients;
 	bool				m_bBounceDCCs;
-	bool				m_bPassHashed;
 	bool				m_bUseClientIP;
 	bool				m_bDenyLoadMod;
 	bool				m_bAdmin;
@@ -247,8 +270,7 @@ protected:
 	bool				m_bIRCConnectEnabled;
 	CIRCSock*			m_pIRCSock;
 
-	CJoinTimer*			m_pJoinTimer;
-	CMiscTimer*			m_pMiscTimer;
+	CUserTimer*			m_pUserTimer;
 
 	vector<CServer*>	m_vServers;
 	vector<CChan*>		m_vChans;
@@ -256,7 +278,7 @@ protected:
 	set<CDCCBounce*>	m_sDCCBounces;
 	set<CDCCSock*>		m_sDCCSocks;
 	set<CString>		m_ssAllowedHosts;
-	unsigned int		m_uServerIdx;
+	unsigned int		m_uServerIdx; ///< Index in m_vServers of our current server + 1
 	unsigned int		m_uBufferCount;
 	unsigned long long      m_uBytesRead;
 	unsigned long long      m_uBytesWritten;
