@@ -8,8 +8,9 @@
 
 #include "ZNCString.h"
 #include "FileUtils.h"
-#include "MD5.h"
 #include "Utils.h"
+#include "MD5.h"
+#include "SHA256.h"
 #include <sstream>
 
 using std::stringstream;
@@ -867,6 +868,26 @@ CString CString::MD5() const {
 	return (const char*) CMD5(*this);
 }
 
+CString CString::SHA256() const {
+	unsigned char digest[SHA256_DIGEST_SIZE];
+	char digest_hex[SHA256_DIGEST_SIZE * 2 + 1];
+	const unsigned char *message = (const unsigned char *) c_str();
+
+	sha256(message, length(), digest);
+
+	sprintf(digest_hex,
+			"%02x%02x%02x%02x%02x%02x%02x%02x"
+			"%02x%02x%02x%02x%02x%02x%02x%02x"
+			"%02x%02x%02x%02x%02x%02x%02x%02x"
+			"%02x%02x%02x%02x%02x%02x%02x%02x",
+			digest[ 0], digest[ 1], digest[ 2], digest[ 3], digest[ 4], digest[ 5], digest[ 6], digest[ 7],
+			digest[ 8], digest[ 9], digest[10], digest[11], digest[12], digest[13], digest[14], digest[15],
+			digest[16], digest[17], digest[18], digest[19], digest[20], digest[21], digest[22], digest[23],
+			digest[24], digest[25], digest[26], digest[27], digest[28], digest[29], digest[30], digest[31]);
+
+	return digest_hex;
+}
+
 #ifdef HAVE_LIBSSL
 CString CString::Encrypt_n(const CString& sPass, const CString& sIvec) {
 	CString sRet;
