@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,6 +9,11 @@
 #include "User.h"
 #include "znc.h"
 
+// If you change these and it breaks, you get to keep the pieces
+#define CHAN_PREFIX_1	"~"
+#define CHAN_PREFIX_1C	'~'
+#define CHAN_PREFIX	CHAN_PREFIX_1 "#"
+
 class CPartylineChannel {
 public:
 	CPartylineChannel(const CString& sName) { m_sName = sName.AsLower(); }
@@ -60,8 +65,8 @@ public:
 		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
 			CUser* pUser = it->second;
 			if (pUser->GetIRCSock()) {
-				if (pUser->GetChanPrefixes().find("~") == CString::npos) {
-					pUser->PutUser(":" + GetIRCServer(pUser) + " 005 " + pUser->GetIRCNick().GetNick() + " CHANTYPES=" + pUser->GetChanPrefixes() + "~ :are supported by this server.");
+				if (pUser->GetChanPrefixes().find(CHAN_PREFIX_1) == CString::npos) {
+					pUser->PutUser(":" + GetIRCServer(pUser) + " 005 " + pUser->GetIRCNick().GetNick() + " CHANTYPES=" + pUser->GetChanPrefixes() + CHAN_PREFIX_1 " :are supported by this server.");
 				}
 			}
 		}
@@ -69,7 +74,7 @@ public:
 		CString sChan;
 		unsigned int a = 0;
 		while (!(sChan = sArgs.Token(a++)).empty()) {
-			if (sChan.Left(2) == "~#") {
+			if (sChan.Left(2) == CHAN_PREFIX) {
 				sChan = sChan.Left(32);
 				m_ssDefaultChans.insert(sChan);
 			}
@@ -124,7 +129,7 @@ public:
 	virtual EModRet OnDeleteUser(CUser& User) {
 		// Loop through each chan
 		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
-			RemoveUser(&User, *it, "KICK", true, "User deleted");
+			RemoveUser(&User, *it, "KICK", true, "User deleted", true);
 		}
 
 		return CONTINUE;
@@ -136,7 +141,7 @@ public:
 			if (uPos != CString::npos) {
 				uPos = sLine.find(" ", uPos);
 
-				sLine.insert(uPos, "~");
+				sLine.insert(uPos, CHAN_PREFIX_1);
 				m_spInjectedPrefixes.insert(m_pUser);
 			}
 		}
@@ -150,7 +155,7 @@ public:
 
 	virtual void OnClientLogin() {
 		if (m_spInjectedPrefixes.find(m_pUser) == m_spInjectedPrefixes.end()) {
-			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 005 " + m_pUser->GetIRCNick().GetNick() + " CHANTYPES=" + m_pUser->GetChanPrefixes() + "~ :are supported by this server.");
+			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 005 " + m_pUser->GetIRCNick().GetNick() + " CHANTYPES=" + m_pUser->GetChanPrefixes() + CHAN_PREFIX_1 " :are supported by this server.");
 		}
 
 		// Make sure this user is in the default channels
@@ -206,11 +211,11 @@ public:
 	}
 
 	virtual EModRet OnUserRaw(CString& sLine) {
-		if (sLine.Equals("WHO ~", false, 5)) {
+		if (sLine.Equals("WHO " CHAN_PREFIX_1, false, 5)) {
 			return HALT;
-		} else if (sLine.Equals("MODE ~", false, 6)) {
+		} else if (sLine.Equals("MODE " CHAN_PREFIX_1, false, 6)) {
 			return HALT;
-		} else if (sLine.Equals("TOPIC ~#", false, 8)) {
+		} else if (sLine.Equals("TOPIC " CHAN_PREFIX, false, 8)) {
 			CString sChannel = sLine.Token(1);
 			CString sTopic = sLine.Token(2, true);
 
@@ -246,11 +251,11 @@ public:
 	}
 
 	virtual EModRet OnUserPart(CString& sChannel, CString& sMessage) {
-		if (sChannel.Left(1) != "~") {
+		if (sChannel.Left(1) != CHAN_PREFIX_1) {
 			return CONTINUE;
 		}
 
-		if (sChannel.Left(2) != "~#") {
+		if (sChannel.Left(2) != CHAN_PREFIX) {
 			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :No such channel");
 			return HALT;
 		}
@@ -268,7 +273,7 @@ public:
 	}
 
 	void RemoveUser(CUser* pUser, CPartylineChannel* pChannel, const CString& sCommand,
-			bool bForce = false, const CString& sMessage = "") {
+			bool bForce = false, const CString& sMessage = "", bool bNickAsTarget = false) {
 		if (!pChannel || !pChannel->IsInChannel(pUser->GetUserName())) {
 			return;
 		}
@@ -289,10 +294,17 @@ public:
 				sHost = pUser->GetIRCNick().GetHost();
 			}
 
-			pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
-					+ pChannel->GetName() + sMsg);
-			PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
-					+ sCmd + pChannel->GetName() + sMsg, false);
+			if (bNickAsTarget) {
+				pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
+						+ pChannel->GetName() + " " + pUser->GetIRCNick().GetNick() + sMsg);
+				PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
+						+ sCmd + pChannel->GetName() + " ?" + pUser->GetUserName() + sMsg, false);
+			} else {
+				pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
+						+ pChannel->GetName() + sMsg);
+				PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
+						+ sCmd + pChannel->GetName() + sMsg, false);
+			}
 
 			if (ssNicks.empty()) {
 				delete pChannel;
@@ -305,12 +317,12 @@ public:
 	}
 
 	virtual EModRet OnUserJoin(CString& sChannel, CString& sKey) {
-		if (sChannel.Left(1) != "~") {
+		if (sChannel.Left(1) != CHAN_PREFIX_1) {
 			return CONTINUE;
 		}
 
-		if (sChannel.Left(2) != "~#") {
-			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :Channels look like ~#znc");
+		if (sChannel.Left(2) != CHAN_PREFIX) {
+			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :Channels look like " CHAN_PREFIX "znc");
 			return HALT;
 		}
 
@@ -343,7 +355,7 @@ public:
 			SendNickList(pUser, ssNicks, pChannel->GetName());
 
 			if (pUser->IsAdmin()) {
-				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), (pUser == m_pUser) ? false : true, pUser);
+				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), false);
 			}
 		}
 	}
@@ -355,7 +367,7 @@ public:
 
 		char cPrefix = sTarget[0];
 
-		if (cPrefix != '~' && cPrefix != '?') {
+		if (cPrefix != CHAN_PREFIX_1C && cPrefix != '?') {
 			return CONTINUE;
 		}
 
@@ -365,7 +377,7 @@ public:
 			sHost = m_pUser->GetIRCNick().GetHost();
 		}
 
-		if (cPrefix == '~') {
+		if (cPrefix == CHAN_PREFIX_1C) {
 			if (FindChannel(sTarget) == NULL) {
 				m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such channel");
 				return HALT;
@@ -455,7 +467,7 @@ public:
 			CUser* pUser = CZNC::Get().FindUser(sUser);
 			CPartylineChannel* pChan;
 
-			if (sChan.Left(2) != "~#") {
+			if (sChan.Left(2) != CHAN_PREFIX) {
 				PutModule("Invalid channel name");
 				return;
 			}
@@ -596,7 +608,7 @@ public:
 	void SendNickList(CUser* pUser, const set<CString>& ssNicks, const CString& sChan) {
 		CString sNickList;
 
-		for (set<CString>::iterator it = ssNicks.begin(); it != ssNicks.end(); it++) {
+		for (set<CString>::const_iterator it = ssNicks.begin(); it != ssNicks.end(); it++) {
 			CUser* pChanUser = CZNC::Get().FindUser(*it);
 
 			if (pChanUser && pChanUser->IsUserAttached()) {
