@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,6 +7,7 @@
  */
 
 #include "User.h"
+#include <algorithm>
 
 class CPerform : public CModule {
 public:
@@ -47,7 +48,7 @@ public:
 			PutModule("Added!");
 			Save();
 		} else if (sCmdName == "del") {
-			u_int iNum = atoi(sCommand.Token(1, true).c_str());
+			u_int iNum = sCommand.Token(1, true).ToUInt();
 			if (iNum > m_vPerform.size() || iNum <= 0) {
 				PutModule("Illegal # Requested");
 				return;
@@ -67,8 +68,22 @@ public:
 					PutModule(CString(i) + ": " + *it);
 			}
 			PutModule(" -- End of List");
+		} else if (sCmdName == "execute") {
+			OnIRCConnected();
+			PutModule("perform commands sent");
+		} else if (sCmdName == "swap") {
+			u_int iNumA = sCommand.Token(1).ToUInt();
+			u_int iNumB = sCommand.Token(2).ToUInt();
+
+			if (iNumA > m_vPerform.size() || iNumA <= 0 || iNumB > m_vPerform.size() || iNumB <= 0) {
+				PutModule("Illegal # Requested");
+			} else {
+				std::iter_swap(m_vPerform.begin() + (iNumA - 1), m_vPerform.begin() + (iNumB - 1));
+				PutModule("Commands Swapped.");
+				Save();
+			}
 		} else {
-			PutModule("Commands: add <command>, del <nr>, list");
+			PutModule("Commands: add <command>, del <nr>, list, execute, swap <nr> <nr>");
 		}
 	}
 
