@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -57,7 +57,7 @@ public:
 	}
 	virtual ~CWatchEntry() {}
 
-	bool IsMatch(const CNick& Nick, const CString& sText, const CString& sSource) {
+	bool IsMatch(const CNick& Nick, const CString& sText, const CString& sSource, const CUser* pUser) {
 		if (IsDisabled()) {
 			return false;
 		}
@@ -70,7 +70,7 @@ public:
 			for (unsigned int a = 0; a < m_vsSources.size(); a++) {
 				const CWatchSource& WatchSource = m_vsSources[a];
 
-				if (sSource.AsLower().WildCmp(Lower(WatchSource.GetSource()))) {
+				if (sSource.AsLower().WildCmp(WatchSource.GetSource().AsLower())) {
 					if (WatchSource.IsNegated()) {
 						return false;
 					} else {
@@ -80,7 +80,14 @@ public:
 			}
 		}
 
-		return (bGoodSource && Nick.GetHostMask().AsLower().WildCmp(Lower(m_sHostMask))) && sText.AsLower().WildCmp(Lower(m_sPattern));
+		if (!bGoodSource)
+			return false;
+		if (!Nick.GetHostMask().AsLower().WildCmp(m_sHostMask.AsLower()))
+			return false;
+		if (!sText.AsLower().WildCmp(pUser->ExpandString(m_sPattern).AsLower()))
+			return false;
+
+		return true;
 	}
 
 	bool operator ==(const CWatchEntry& WatchEntry) {
@@ -154,13 +161,14 @@ class CWatcherMod : public CModule {
 public:
 	MODCONSTRUCTOR(CWatcherMod) {
 		m_Buffer.SetLineCount(500);
+		Load();
 	}
 
-	virtual ~CWatcherMod() {
-	}
+	virtual ~CWatcherMod() {}
 
 	virtual void OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) {
-		Process(OpNick, "* " + OpNick.GetNick() + " sets mode: " + sModes + " " + sArgs + " on " + Channel.GetName(), Channel.GetName());
+		Process(OpNick, "* " + OpNick.GetNick() + " sets mode: " + sModes + " " +
+			sArgs + " on " + Channel.GetName(), Channel.GetName());
 	}
 
 	virtual void OnClientLogin() {
@@ -173,19 +181,23 @@ public:
 	}
 
 	virtual void OnKick(const CNick& OpNick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) {
-		Process(OpNick, "* " + OpNick.GetNick() + " kicked " + sKickedNick + " from " + Channel.GetName() + " because [" + sMessage + "]", Channel.GetName());
+		Process(OpNick, "* " + OpNick.GetNick() + " kicked " + sKickedNick + " from " + 
+			Channel.GetName() + " because [" + sMessage + "]", Channel.GetName());
 	}
 
 	virtual void OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) {
-		Process(Nick, "* Quits: " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") (" + sMessage + ")", "");
+		Process(Nick, "* Quits: " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") "
+			"(" + sMessage + ")", "");
 	}
 
 	virtual void OnJoin(const CNick& Nick, CChan& Channel) {
-		Process(Nick, "* " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") joins " + Channel.GetName(), Channel.GetName());
+		Process(Nick, "* " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") joins " +
+			Channel.GetName(), Channel.GetName());
 	}
 
 	virtual void OnPart(const CNick& Nick, CChan& Channel) {
-		Process(Nick, "* " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") parts " + Channel.GetName(), Channel.GetName());
+		Process(Nick, "* " + Nick.GetNick() + " (" + Nick.GetIdent() + "@" + Nick.GetHost() + ") parts " +
+			Channel.GetName(), Channel.GetName());
 	}
 
 	virtual void OnNick(const CNick& OldNick, const CString& sNewNick, const vector<CChan*>& vChans) {
@@ -203,7 +215,8 @@ public:
 	}
 
 	virtual EModRet OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage) {
-		Process(Nick, "* CTCP: " + Nick.GetNick() + " [" + sMessage + "] to [" + Channel.GetName() + "]", Channel.GetName());
+		Process(Nick, "* CTCP: " + Nick.GetNick() + " [" + sMessage + "] to "
+			"[" + Channel.GetName() + "]", Channel.GetName());
 		return CONTINUE;
 	}
 
@@ -243,7 +256,7 @@ public:
 			if (sTok == "*") {
 				SetDisabled(~0, false);
 			} else {
-				SetDisabled(atoi(sTok.c_str()), false);
+				SetDisabled(sTok.ToUInt(), false);
 			}
 		} else if (sCmdName.Equals("DISABLE")) {
 			CString sTok = sCommand.Token(1);
@@ -251,23 +264,24 @@ public:
 			if (sTok == "*") {
 				SetDisabled(~0, true);
 			} else {
-				SetDisabled(atoi(sTok.c_str()), true);
+				SetDisabled(sTok.ToUInt(), true);
 			}
 		} else if (sCmdName.Equals("SETSOURCES")) {
-			SetSources(atoi(sCommand.Token(1).c_str()), sCommand.Token(2, true));
+			SetSources(sCommand.Token(1).ToUInt(), sCommand.Token(2, true));
 		} else if (sCmdName.Equals("CLEAR")) {
 			m_lsWatchers.clear();
 			PutModule("All entries cleared.");
+			Save();
 		} else if (sCmdName.Equals("BUFFER")) {
 			CString sCount = sCommand.Token(1);
 
 			if (sCount.size()) {
-				m_Buffer.SetLineCount(atoi(sCount.c_str()));
+				m_Buffer.SetLineCount(sCount.ToUInt());
 			}
 
 			PutModule("Buffer count is set to [" + CString(m_Buffer.GetLineCount()) + "]");
 		} else if (sCmdName.Equals("DEL")) {
-			Remove(atoi(sCommand.Token(1).c_str()));
+			Remove(sCommand.Token(1).ToUInt());
 		} else {
 			PutModule("Unknown command: [" + sCmdName + "]");
 		}
@@ -278,11 +292,13 @@ private:
 		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
 			CWatchEntry& WatchEntry = *it;
 
-			if (WatchEntry.IsMatch(Nick, sMessage, sSource)) {
+			if (WatchEntry.IsMatch(Nick, sMessage, sSource, m_pUser)) {
 				if (m_pUser->IsUserAttached()) {
-					m_pUser->PutUser(":" + WatchEntry.GetTarget() + "!watch@znc.in PRIVMSG " + m_pUser->GetCurNick() + " :" + sMessage);
+					m_pUser->PutUser(":" + WatchEntry.GetTarget() + "!watch@znc.in PRIVMSG " +
+							m_pUser->GetCurNick() + " :" + sMessage);
 				} else {
-					m_Buffer.AddLine(":" + WatchEntry.GetTarget() + "!watch@znc.in PRIVMSG ", " :" + sMessage);
+					m_Buffer.AddLine(":" + WatchEntry.GetTarget() + "!watch@znc.in PRIVMSG ",
+							" :" + m_pUser->AddTimestamp(sMessage));
 				}
 			}
 		}
@@ -295,6 +311,7 @@ private:
 			}
 
 			PutModule(((bDisabled) ? "Disabled all entries." : "Enabled all entries."));
+			Save();
 			return;
 		}
 
@@ -310,6 +327,7 @@ private:
 
 		(*it).SetDisabled(bDisabled);
 		PutModule("Id " + CString(uIdx +1) + ((bDisabled) ? " Disabled" : " Enabled"));
+		Save();
 	}
 
 	void List() {
@@ -356,10 +374,12 @@ private:
 		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++, uIdx++) {
 			CWatchEntry& WatchEntry = *it;
 
-			PutModule("/msg " + GetModNick() + " ADD " + WatchEntry.GetHostMask() + " " + WatchEntry.GetTarget() + " " + WatchEntry.GetPattern());
+			PutModule("/msg " + GetModNick() + " ADD " + WatchEntry.GetHostMask() + " " +
+				WatchEntry.GetTarget() + " " + WatchEntry.GetPattern());
 
 			if (WatchEntry.GetSourcesStr().size()) {
-				PutModule("/msg " + GetModNick() + " SETSOURCES " + CString(uIdx) + " " + WatchEntry.GetSourcesStr());
+				PutModule("/msg " + GetModNick() + " SETSOURCES " + CString(uIdx) + " " +
+					WatchEntry.GetSourcesStr());
 			}
 
 			if (WatchEntry.IsDisabled()) {
@@ -383,6 +403,7 @@ private:
 
 		(*it).SetSources(sSources);
 		PutModule("Sources set for Id " + CString(uIdx +1) + ".");
+		Save();
 	}
 
 	void Remove(unsigned int uIdx) {
@@ -398,6 +419,7 @@ private:
 
 		m_lsWatchers.erase(it);
 		PutModule("Id " + CString(uIdx +1) + " Removed.");
+		Save();
 	}
 
 	void Help() {
@@ -465,7 +487,8 @@ private:
 			}
 
 			if (!bExists) {
-				sMessage = "Adding entry: [" + WatchEntry.GetHostMask() + "] watching for [" + WatchEntry.GetPattern() + "] -> [" + WatchEntry.GetTarget() + "]";
+				sMessage = "Adding entry: [" + WatchEntry.GetHostMask() + "] watching for "
+					"[" + WatchEntry.GetPattern() + "] -> [" + WatchEntry.GetTarget() + "]";
 				m_lsWatchers.push_back(WatchEntry);
 			}
 		} else {
@@ -477,6 +500,56 @@ private:
 		} else {
 			PutModule(sMessage);
 		}
+		Save();
+	}
+
+	void Save() {
+		ClearNV(false);
+		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
+			CWatchEntry& WatchEntry = *it;
+			CString sSave;
+
+			sSave  = WatchEntry.GetHostMask() + "\n";
+			sSave += WatchEntry.GetTarget() + "\n";
+			sSave += WatchEntry.GetPattern() + "\n";
+			sSave += (WatchEntry.IsDisabled() ? "disabled\n" : "enabled\n");
+			sSave += WatchEntry.GetSourcesStr();
+			// Without this, loading fails if GetSourcesStr()
+			// returns an empty string
+			sSave += " ";
+
+			SetNV(sSave, "", false);
+		}
+
+		SaveRegistry();
+	}
+
+	void Load() {
+		// Just to make sure we dont mess up badly
+		m_lsWatchers.clear();
+
+		bool bWarn = false;
+
+		for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
+			VCString vList;
+			it->first.Split("\n", vList);
+
+			if (vList.size() != 5) {
+				bWarn = true;
+				continue;
+			}
+
+			CWatchEntry WatchEntry(vList[0], vList[1], vList[2]);
+			if (vList[3].Equals("disabled"))
+				WatchEntry.SetDisabled(true);
+			else
+				WatchEntry.SetDisabled(false);
+			WatchEntry.SetSources(vList[4]);
+			m_lsWatchers.push_back(WatchEntry);
+		}
+
+		if (bWarn)
+			PutModule("WARNING: malformed entry found while loading");
 	}
 
 	list<CWatchEntry>	m_lsWatchers;
