@@ -25,7 +25,7 @@ public:
 
 	virtual ~CWebAdminAuth() {}
 
-	void SetWebAdminSock(CWebAdminSock* pWebAdminSock) { m_pWebAdminSock = pWebAdminSock; }
+	void Invalidate() { m_pWebAdminSock = NULL; CAuthBase::Invalidate(); }
 	void AcceptedLogin(CUser& User);
 	void RefusedLogin(const CString& sReason);
 private:
@@ -228,9 +228,11 @@ public:
 				|| sLine.WildCmp("POST * HTTP/1.?")) {
 			CWebAdminSock* pSock = new CWebAdminSock(this);
 			CZNC::Get().GetManager().SwapSockByAddr(pSock, pClient);
-			// And don't forget to give it some sane name / timeout
+
+			// And don't forget to give it some sane settings again
 			pSock->SetSockName("WebAdmin::Client");
 			pSock->SetTimeout(120);
+			pSock->SetMaxBufferThreshold(10240);
 
 			// TODO can we somehow get rid of this?
 			pSock->ReadLine(sLine);
@@ -357,7 +359,7 @@ CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule, const CString& sHostname, un
 CWebAdminSock::~CWebAdminSock() {
 	if (!m_spAuth.IsNull()) {
 		CWebAdminAuth* pAuth = (CWebAdminAuth*) &(*m_spAuth);
-		pAuth->SetWebAdminSock(NULL);
+		pAuth->Invalidate();
 	}
 }
 
@@ -1037,8 +1039,8 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 	if (!sArg.empty()) {
 		CString sSalt = CUtils::GetSalt();
-		CString sHash = CUtils::SaltedHash(sArg, sSalt);
-		pNewUser->SetPass(sHash, true, sSalt);
+		CString sHash = CUser::SaltedHash(sArg, sSalt);
+		pNewUser->SetPass(sHash, CUser::HASH_DEFAULT, sSalt);
 	}
 
 	VCString vsArgs;
@@ -1068,43 +1070,6 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		pNewUser->AddCTCPReply(sReply.Token(0).Trim_n(), sReply.Token(1, true).Trim_n());
 	}
 
-	if (IsAdmin() || (pUser && !pUser->DenyLoadMod())) {
-		GetParamValues("loadmod", vsArgs);
-
-		for (a = 0; a < vsArgs.size(); a++) {
-			CString sModRet;
-			CString sModName = vsArgs[a].TrimRight_n("\r");
-
-			if (!sModName.empty()) {
-				CString sArgs = GetParam("modargs_" + sModName);
-
-				try {
-					if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
-						DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
-					}
-				} catch (...) {
-					DEBUG("Unable to load module [" << sModName << "] [" << sArgs << "]");
-				}
-			}
-		}
-	} else if (pUser) {
-		CModules& Modules = pUser->GetModules();
-
-		for (a = 0; a < Modules.size(); a++) {
-			CString sModName = Modules[a]->GetModName();
-			CString sArgs = Modules[a]->GetArgs();
-			CString sModRet;
-
-			try {
-				if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
-					DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
-				}
-			} catch (...) {
-				DEBUG("Unable to load module [" << sModName << "]");
-			}
-		}
-	}
-
 	sArg = GetParam("nick"); if (!sArg.empty()) { pNewUser->SetNick(sArg); }
 	sArg = GetParam("altnick"); if (!sArg.empty()) { pNewUser->SetAltNick(sArg); }
 	sArg = GetParam("statusprefix"); if (!sArg.empty()) { pNewUser->SetStatusPrefix(sArg); }
@@ -1142,7 +1107,9 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		pNewUser->SetDenySetVHost(pUser->DenySetVHost());
 	}
 
-	if (pUser && pUser != CZNC::Get().FindUser(GetUser())) {
+	// If pUser is not NULL, we are editing an existing user.
+	// Users must not be able to change their own admin flag.
+	if (pUser != CZNC::Get().FindUser(GetUser())) {
 		pNewUser->SetAdmin(GetParam("isadmin").ToBool());
 	} else if (pUser) {
 		pNewUser->SetAdmin(pUser->IsAdmin());
@@ -1154,6 +1121,43 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		pNewUser->AddChan(sChan.TrimRight_n("\r"), GetParam("save_" + sChan).ToBool());
 	}
 
+	if (IsAdmin() || (pUser && !pUser->DenyLoadMod())) {
+		GetParamValues("loadmod", vsArgs);
+
+		for (a = 0; a < vsArgs.size(); a++) {
+			CString sModRet;
+			CString sModName = vsArgs[a].TrimRight_n("\r");
+
+			if (!sModName.empty()) {
+				CString sArgs = GetParam("modargs_" + sModName);
+
+				try {
+					if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
+						DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
+					}
+				} catch (...) {
+					DEBUG("Unable to load module [" << sModName << "] [" << sArgs << "]");
+				}
+			}
+		}
+	} else if (pUser) {
+		CModules& Modules = pUser->GetModules();
+
+		for (a = 0; a < Modules.size(); a++) {
+			CString sModName = Modules[a]->GetModName();
+			CString sArgs = Modules[a]->GetArgs();
+			CString sModRet;
+
+			try {
+				if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
+					DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
+				}
+			} catch (...) {
+				DEBUG("Unable to load module [" << sModName << "]");
+			}
+		}
+	}
+
 	return pNewUser;
 }
 
