@@ -27,7 +27,7 @@ namespace
 
 CZNC::CZNC() {
 	if (!InitCsocket()) {
-		CUtils::PrintError("Failed to initialize Csocket!");
+		CUtils::PrintError("Could not initialize Csocket!");
 		exit(-1);
 	}
 
@@ -41,7 +41,7 @@ CZNC::CZNC() {
 	m_uBytesRead = 0;
 	m_uBytesWritten = 0;
 	m_pConnectUserTimer = NULL;
-	m_bNeedRehash = false;
+	m_eConfigState = ECONFIG_NOTHING;
 	m_TimeStarted = time(NULL);
 }
 
@@ -165,6 +165,14 @@ bool CZNC::ConnectUser(CUser *pUser) {
 	}
 #endif
 
+	MODULECALL(OnIRCConnecting(pIRCSock), pUser, NULL,
+		DEBUG("Some module aborted the connection attempt");
+		pUser->PutStatus("Some module aborted the connection attempt");
+		ReleaseISpoof();
+		delete pIRCSock;
+		return false;
+	);
+
 	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 120, bSSL, pUser->GetVHost(), pIRCSock)) {
 		ReleaseISpoof();
 		pUser->PutStatus("Unable to connect. (Bad host?)");
@@ -220,8 +228,9 @@ void CZNC::Loop() {
 	while (true) {
 		CString sError;
 
-		if (GetNeedRehash()) {
-			SetNeedRehash(false);
+		switch (GetConfigState()) {
+		case ECONFIG_NEED_REHASH:
+			SetConfigState(ECONFIG_NOTHING);
 
 			if (RehashConfig(sError)) {
 				Broadcast("Rehashing succeeded", true);
@@ -229,6 +238,18 @@ void CZNC::Loop() {
 				Broadcast("Rehashing failed: " + sError, true);
 				Broadcast("ZNC is in some possibly inconsistent state!", true);
 			}
+			break;
+		case ECONFIG_NEED_WRITE:
+			SetConfigState(ECONFIG_NOTHING);
+
+			if (WriteConfig()) {
+				Broadcast("Writing the config suceeded", true);
+			} else {
+				Broadcast("Writing the config file failed", true);
+			}
+			break;
+		case ECONFIG_NOTHING:
+			break;
 		}
 
 		// Check for users that need to be deleted
@@ -266,7 +287,13 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 			return false;
 		}
 
-		CString sData = m_sISpoofFormat.Token(0, false, "%") + pUser->GetIdent() + m_sISpoofFormat.Token(1, true, "%");
+		CString sData = pUser->ExpandString(m_sISpoofFormat);
+
+		// If the format doesn't contain anything expandable, we'll
+		// assume this is an "old"-style format string.
+		if (sData == m_sISpoofFormat) {
+			sData.Replace("%", pUser->GetIdent());
+		}
 		m_pISpoofLockFile->Write(sData + "\n");
 	}
 	return true;
@@ -706,7 +733,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		vsLines.push_back("<User " + sUser + ">");
 		CString sSalt;
 		sAnswer = CUtils::GetSaltedHashPass(sSalt);
-		vsLines.push_back("\tPass       = md5#" + sAnswer + "#" + sSalt + "#");
+		vsLines.push_back("\tPass       = " + CUtils::sDefaultHash + "#" + sAnswer + "#" + sSalt + "#");
 
 		if (CUtils::GetBoolInput("Would you like this user to be an admin?", bFirstUser)) {
 			vsLines.push_back("\tAdmin      = true");
@@ -1236,24 +1263,29 @@ bool CZNC::DoRehash(CString& sError)
 						// Pass = <plain text>
 						// Pass = <md5 hash> -
 						// Pass = plain#<plain text>
-						// Pass = md5#<md5 hash>
-						// Pass = md5#<salted md5 hash>#<salt>#
-						// The last one is the md5 hash of 'password' + 'salt'
+						// Pass = <hash name>#<hash>
+						// Pass = <hash name>#<salted hash>#<salt>#
+						// 'Salted hash' means hash of 'password' + 'salt'
+						// Possible hashes are md5 and sha256
 						if (sValue.Right(1) == "-") {
 							sValue.RightChomp();
 							sValue.Trim();
-							pUser->SetPass(sValue, true);
+							pUser->SetPass(sValue, CUser::HASH_MD5);
 						} else {
 							CString sMethod = sValue.Token(0, false, "#");
 							CString sPass = sValue.Token(1, true, "#");
-							if (sMethod == "md5") {
+							if (sMethod == "md5" || sMethod == "sha256") {
+								CUser::eHashType type = CUser::HASH_MD5;
+								if (sMethod == "sha256")
+									type = CUser::HASH_SHA256;
+
 								CString sSalt = sPass.Token(1, false, "#");
 								sPass = sPass.Token(0, false, "#");
-								pUser->SetPass(sPass, true, sSalt);
+								pUser->SetPass(sPass, type, sSalt);
 							} else if (sMethod == "plain") {
-								pUser->SetPass(sPass, false);
+								pUser->SetPass(sPass, CUser::HASH_NONE);
 							} else {
-								pUser->SetPass(sValue, false);
+								pUser->SetPass(sValue, CUser::HASH_NONE);
 							}
 						}
 
@@ -1300,6 +1332,9 @@ bool CZNC::DoRehash(CString& sError)
 					} else if (sName.Equals("VHost")) {
 						pUser->SetVHost(sValue);
 						continue;
+					} else if (sName.Equals("DCCVHost")) {
+						pUser->SetDCCVHost(sValue);
+						continue;
 					} else if (sName.Equals("Allow")) {
 						pUser->AddAllowedHost(sValue);
 						continue;
@@ -1672,35 +1707,6 @@ void CZNC::Broadcast(const CString& sMessage, bool bAdminOnly,
 	}
 }
 
-bool CZNC::FindModPath(const CString& sModule, CString& sModPath,
-		CString& sDataPath) const {
-	CString sMod = sModule;
-	CString sDir = sMod;
-	if (sModule.find(".") == CString::npos)
-		sMod += ".so";
-
-	sDataPath = GetCurPath() + "/modules/";
-	sModPath = sDataPath + sMod;
-
-	if (!CFile::Exists(sModPath)) {
-		sDataPath = GetModPath() + "/";
-		sModPath = sDataPath + sMod;
-
-		if (!CFile::Exists(sModPath)) {
-			sDataPath = _DATADIR_ + CString("/");
-			sModPath = _MODDIR_ + CString("/") + sMod;
-
-			if (!CFile::Exists(sModPath)) {
-				return false;
-			}
-		}
-	}
-
-	sDataPath += sDir;
-
-	return true;
-}
-
 CUser* CZNC::FindUser(const CString& sUsername) {
 	map<CString,CUser*>::iterator it = m_msUsers.find(sUsername);
 
@@ -1818,7 +1824,18 @@ public:
 		// Don't wait iSecs seconds for first timer run
 		m_bRunOnNextCall = true;
 	}
-	virtual ~CConnectUserTimer() {}
+	virtual ~CConnectUserTimer() {
+		// This is only needed when ZNC shuts down:
+		// CZNC::~CZNC() sets its CConnectUserTimer pointer to NULL and
+		// calls the manager's Cleanup() which destroys all sockets and
+		// timers. If something calls CZNC::EnableConnectUser() here
+		// (e.g. because a CIRCSock is destroyed), the socket manager
+		// deletes that timer almost immediately, but CZNC now got a
+		// dangling pointer to this timer which can crash later on.
+		//
+		// Unlikely but possible ;)
+		CZNC::Get().LeakConnectUser(this);
+	}
 
 protected:
 	virtual void RunJob() {
@@ -1894,3 +1911,8 @@ void CZNC::DisableConnectUser() {
 	m_pConnectUserTimer->Stop();
 	m_pConnectUserTimer = NULL;
 }
+
+void CZNC::LeakConnectUser(CConnectUserTimer *pTimer) {
+	if (m_pConnectUserTimer == pTimer)
+		m_pConnectUserTimer = NULL;
+}
