@@ -28,6 +28,12 @@ public:
 	CZNC();
 	~CZNC();
 
+	enum ConfigState {
+		ECONFIG_NOTHING,
+		ECONFIG_NEED_REHASH,
+		ECONFIG_NEED_WRITE
+	};
+
 	void DeleteUsers();
 	void Loop();
 	bool WriteISpoof(CUser* pUser);
@@ -49,10 +55,6 @@ public:
 	static CString GetVersion();
 	static CString GetTag(bool bIncludeVersion = true);
 	CString GetUptime() const;
-	// This returns the path to the .so and to the data dir
-	// which is where static data (webadmin skins) are saved
-	bool FindModPath(const CString& sModule, CString& sModPath,
-			CString& sDataPath) const;
 	void ClearVHosts();
 	bool AddVHost(const CString& sHost);
 	bool RemVHost(const CString& sHost);
@@ -79,14 +81,14 @@ public:
 	void AuthUser(CSmartPtr<CAuthBase> AuthClass);
 
 	// Setters
-	void SetNeedRehash(bool b) { m_bNeedRehash = b; }
+	void SetConfigState(enum ConfigState e) { m_eConfigState = e; }
 	void SetStatusPrefix(const CString& s) { m_sStatusPrefix = (s.empty()) ? "*" : s; }
 	void SetISpoofFile(const CString& s) { m_sISpoofFile = s; }
 	void SetISpoofFormat(const CString& s) { m_sISpoofFormat = (s.empty()) ? "global { reply \"%\" }" : s; }
 	// !Setters
 
 	// Getters
-	bool GetNeedRehash() const { return m_bNeedRehash; }
+	enum ConfigState GetConfigState() const { return m_eConfigState; }
 	CSockManager& GetManager() { return m_Manager; }
 	const CSockManager& GetManager() const { return m_Manager; }
 #ifdef _MODULES
@@ -129,6 +131,9 @@ public:
 	void EnableConnectUser();
 	void DisableConnectUser();
 
+	// Never call this unless you are CConnectUserTimer::~CConnectUserTimer()
+	void LeakConnectUser(CConnectUserTimer *pTimer);
+
 private:
 	bool DoRehash(CString& sError);
 	// Returns true if something was done
@@ -137,7 +142,7 @@ private:
 protected:
 	time_t				m_TimeStarted;
 
-	bool				m_bNeedRehash;
+	enum ConfigState		m_eConfigState;
 	vector<CListener*>		m_vpListeners;
 	map<CString,CUser*>		m_msUsers;
 	map<CString,CUser*>		m_msDelUsers;
